from ._news_data_provider_layer import NewsDataProviderLayer
from ..._tools import create_repr

from ..._content_type import ContentType


class Definition(NewsDataProviderLayer):
    """
    This class describes parameters to retrieve data for news story.

    Parameters
    ----------
    story_id : str
        News Story ID.

    Examples
    --------
    >>> from refinitiv.data.content import news
    >>> definition = news.story.Definition("urn:newsml:reuters.com:20201026:nPt6BSyBh")
    """

    def __init__(
        self,
        story_id: str,
    ):
        super().__init__(
            data_type=ContentType.NEWS_STORY_RDP,
            story_id=story_id,
        )
        self.story_id = story_id

    def __repr__(self):
        return create_repr(
            self,
            middle_path="story",
            content=f"{{story_id='{self.story_id}'}}",
        )



"""Builtin discovery search templates

Created as JSON configuration by Product Owner and Search Team.
Converted to code using code-generation.
Please, don't do manual changes to it, they may be lost after the next code-generation
"""

import pandas as pd

from .search import DiscoverySearchTemplate
from .namespaces import Namespace


class RICCategoryTemplate(DiscoverySearchTemplate):
    """Returns the category for a given RIC"""

    def __init__(self):
        super().__init__(
            name="RICCategory",
            pass_through_defaults={
                "view": "QuotesAndSTIRs",
                "select": "RCSAssetCategoryLeaf",
                "top": 1000,
            },
            filter="RIC eq '#{ric}'",
        )

    def _search_kwargs(self, *, ric, **kwargs) -> dict:
        return super()._search_kwargs(ric=ric, **kwargs)

    def search(self, *, ric, **kwargs) -> pd.DataFrame:
        """Launch search, get DataFrame

        Parameters
        ----------
        ric
            The RIC for which to search the category.

        Returns
        -------
        DataFrame
        Default columns: RCSAssetCategoryLeaf
        """
        return super().search(ric=ric, **kwargs)


class UnderlyingRICToOptionTemplate(DiscoverySearchTemplate):
    """Simple search for options on any underlying"""

    def __init__(self):
        super().__init__(
            name="UnderlyingRICToOption",
            pass_through_defaults={
                "view": "SearchAll",
                "select": "RIC,DTSubjectName,ExpiryDateString,StrikePrice,CallPutOption,ContractType,Currency",
                "order_by": "ExpiryDate,StrikePrice,CallPutOption",
                "top": 1000,
            },
            optional_placeholders=(
                "strike_price",
                "expiry_date",
                "put_call",
                "contract_type",
            ),
            filter="UnderlyingQuoteRIC eq '#{ric}' and RCSAssetCategoryLeaf eq 'Option' and IsChain eq false {{if strike_price is not none}} and StrikePrice eq #{strike_price}{{endif}} {{if expiry_date}} and ExpiryDate eq #{expiry_date}{{endif}} {{if put_call}} and CallPutOption eq '#{put_call}'{{endif}} {{if contract_type}} and ContractType eq '#{contract_type}'{{endif}}",
        )

    def _search_kwargs(
        self,
        *,
        ric,
        contract_type=None,
        expiry_date=None,
        put_call=None,
        strike_price=None,
        **kwargs,
    ) -> dict:
        return super()._search_kwargs(
            contract_type=contract_type,
            expiry_date=expiry_date,
            put_call=put_call,
            ric=ric,
            strike_price=strike_price,
            **kwargs,
        )

    def search(
        self,
        *,
        ric,
        contract_type=None,
        expiry_date=None,
        put_call=None,
        strike_price=None,
        **kwargs,
    ) -> pd.DataFrame:
        """Launch search, get DataFrame

        Parameters
        ----------
        ric

        strike_price

        expiry_date

        put_call

        contract_type

        Returns
        -------
        DataFrame
        Default columns: RIC, DTSubjectName, ExpiryDateString, StrikePrice, CallPutOption, ContractType, Currency
        """
        return super().search(
            contract_type=contract_type,
            expiry_date=expiry_date,
            put_call=put_call,
            ric=ric,
            strike_price=strike_price,
            **kwargs,
        )


class UnderlyingRICToFutureTemplate(DiscoverySearchTemplate):
    """Simple search for futures on any underlying"""

    def __init__(self):
        super().__init__(
            name="UnderlyingRICToFuture",
            pass_through_defaults={
                "view": "SearchAll",
                "select": "RIC,DTSubjectName,ExpiryDateString,ContractType,Currency",
                "order_by": "ExpiryDate",
                "top": 1000,
            },
            filter="UnderlyingQuoteRIC eq '#{ric}' and RCSAssetCategoryLeaf eq 'Future' and IsChain eq false and AssetStateName eq 'Active'",
        )

    def _search_kwargs(self, *, ric, **kwargs) -> dict:
        return super()._search_kwargs(ric=ric, **kwargs)

    def search(self, *, ric, **kwargs) -> pd.DataFrame:
        """Launch search, get DataFrame

        Parameters
        ----------
        ric
            The underlying instrument for which to search for futures.

        Returns
        -------
        DataFrame
        Default columns: RIC, DTSubjectName, ExpiryDateString, ContractType, Currency
        """
        return super().search(ric=ric, **kwargs)


class RICToIssuerTemplate(DiscoverySearchTemplate):
    """Find issuer of a RIC"""

    def __init__(self):
        super().__init__(
            name="RICToIssuer",
            pass_through_defaults={
                "view": "SearchAll",
                "select": "DTSubjectName,RCSIssuerCountryLeaf,IssuerOAPermID,PrimaryRIC",
                "top": 1000,
            },
            filter="RIC eq '#{ric}'",
        )

    def _search_kwargs(self, *, ric, **kwargs) -> dict:
        return super()._search_kwargs(ric=ric, **kwargs)

    def search(self, *, ric, **kwargs) -> pd.DataFrame:
        """Launch search, get DataFrame

        Parameters
        ----------
        ric
            The RIC for which to search the issuer.

        Returns
        -------
        DataFrame
        Default columns: DTSubjectName, RCSIssuerCountryLeaf, IssuerOAPermID, PrimaryRIC
        """
        return super().search(ric=ric, **kwargs)


class FutureRicToFutureTemplate(DiscoverySearchTemplate):
    """From one future RIC, find all other futures on its underlying"""

    def __init__(self):
        super().__init__(
            name="FutureRicToFuture",
            pass_through_defaults={
                "view": "SearchAll",
                "select": "RicRoot,RIC,DTSubjectName,ExpiryDateString,ContractType,Currency",
                "order_by": "GrossTonnage desc",
                "top": 1000,
            },
            filter="RicRoot eq '#{_.GetRoot.RicRoot.0}' and RCSAssetCategoryLeaf xeq 'Equity Future' and IsChain eq false and AssetStateName eq 'Active' and DisplayType ne 'CONTN'",
            ns=Namespace(
                _=Namespace(
                    GetRoot=DiscoverySearchTemplate(
                        pass_through_defaults={
                            "view": "SearchAll",
                            "select": "RicRoot",
                            "top": 1,
                        },
                        filter="RIC eq '#{ric}'",
                    )
                )
            ),
        )

    def _search_kwargs(self, **kwargs) -> dict:
        return super()._search_kwargs(**kwargs)

    def search(self, **kwargs) -> pd.DataFrame:
        """Launch search, get DataFrame

        Parameters
        ----------
        ric

        Returns
        -------
        DataFrame
        Default columns: RicRoot, RIC, DTSubjectName, ExpiryDateString, ContractType, Currency
        """
        return super().search(**kwargs)


class OrganisationPermIDToUPTemplate(DiscoverySearchTemplate):
    """Find ultimate parent of an organisation"""

    def __init__(self):
        super().__init__(
            name="OrganisationPermIDToUP",
            pass_through_defaults={
                "view": "SearchAll",
                "select": "UltimateParentCompanyOAPermID,UltimateParentOrganisationName",
                "top": 1000,
            },
            filter="OAPermID eq '#{entity_id}'",
        )

    def _search_kwargs(self, *, entity_id, **kwargs) -> dict:
        return super()._search_kwargs(entity_id=entity_id, **kwargs)

    def search(self, *, entity_id, **kwargs) -> pd.DataFrame:
        """Launch search, get DataFrame

        Parameters
        ----------
        entity_id
            PermID of the organisation.

        Returns
        -------
        DataFrame
        Default columns: UltimateParentCompanyOAPermID, UltimateParentOrganisationName
        """
        return super().search(entity_id=entity_id, **kwargs)



from typing import Optional, TYPE_CHECKING, List

from ._curve_definition_pricing import (
    CrossCurrencyCurveDefinitionPricing,
)
from ... import ZcCurveDefinition
from ...._object_definition import ObjectDefinition
from ......_tools import try_copy_to_list

if TYPE_CHECKING:
    from ......_types import OptStr, OptBool, OptStrings


class FxForwardCurveDefinition(ObjectDefinition):
    """
    Generates the Cross Currency curves for the definitions provided

    Parameters
    ----------
    cross_currency_definitions : list of CrossCurrencyCurveDefinitionPricing, optional

    curve_tenors : string, optional
        List of user-defined curve tenors or dates to be computed.
    interest_rate_curve_definitions : list of ZcCurveDefinition, optional

    base_currency : str, optional

    base_index_name : str, optional

    is_non_deliverable : bool, optional
        If true, non deliverable cross currency definition is used. default value is
        false
    pivot_currency : str, optional

    pivot_index_name : str, optional

    quoted_currency : str, optional

    quoted_index_name : str, optional

    """

    def __init__(
        self,
        cross_currency_definitions: Optional[
            List[CrossCurrencyCurveDefinitionPricing]
        ] = None,
        curve_tenors: "OptStrings" = None,
        interest_rate_curve_definitions: Optional[List[ZcCurveDefinition]] = None,
        base_currency: "OptStr" = None,
        base_index_name: "OptStr" = None,
        is_non_deliverable: "OptBool" = None,
        pivot_currency: "OptStr" = None,
        pivot_index_name: "OptStr" = None,
        quoted_currency: "OptStr" = None,
        quoted_index_name: "OptStr" = None,
    ) -> None:
        super().__init__()
        self.cross_currency_definitions = try_copy_to_list(cross_currency_definitions)
        self.curve_tenors = curve_tenors
        self.interest_rate_curve_definitions = try_copy_to_list(
            interest_rate_curve_definitions
        )
        self.base_currency = base_currency
        self.base_index_name = base_index_name
        self.is_non_deliverable = is_non_deliverable
        self.pivot_currency = pivot_currency
        self.pivot_index_name = pivot_index_name
        self.quoted_currency = quoted_currency
        self.quoted_index_name = quoted_index_name

    @property
    def cross_currency_definitions(self):
        """
        :return: list CrossCurrencyCurveDefinitionPricing
        """
        return self._get_list_parameter(
            CrossCurrencyCurveDefinitionPricing, "crossCurrencyDefinitions"
        )

    @cross_currency_definitions.setter
    def cross_currency_definitions(self, value):
        self._set_list_parameter(
            CrossCurrencyCurveDefinitionPricing, "crossCurrencyDefinitions", value
        )

    @property
    def curve_tenors(self):
        """
        List of user-defined curve tenors or dates to be computed.
        :return: list string
        """
        return self._get_list_parameter(str, "curveTenors")

    @curve_tenors.setter
    def curve_tenors(self, value):
        self._set_list_parameter(str, "curveTenors", value)

    @property
    def interest_rate_curve_definitions(self):
        """
        :return: list ZcCurveDefinition
        """
        return self._get_list_parameter(
            ZcCurveDefinition, "interestRateCurveDefinitions"
        )

    @interest_rate_curve_definitions.setter
    def interest_rate_curve_definitions(self, value):
        self._set_list_parameter(
            ZcCurveDefinition, "interestRateCurveDefinitions", value
        )

    @property
    def base_currency(self):
        """
        :return: str
        """
        return self._get_parameter("baseCurrency")

    @base_currency.setter
    def base_currency(self, value):
        self._set_parameter("baseCurrency", value)

    @property
    def base_index_name(self):
        """
        :return: str
        """
        return self._get_parameter("baseIndexName")

    @base_index_name.setter
    def base_index_name(self, value):
        self._set_parameter("baseIndexName", value)

    @property
    def is_non_deliverable(self):
        """
        If true, non deliverable cross currency definition is used. default value is
        false
        :return: bool
        """
        return self._get_parameter("isNonDeliverable")

    @is_non_deliverable.setter
    def is_non_deliverable(self, value):
        self._set_parameter("isNonDeliverable", value)

    @property
    def pivot_currency(self):
        """
        :return: str
        """
        return self._get_parameter("pivotCurrency")

    @pivot_currency.setter
    def pivot_currency(self, value):
        self._set_parameter("pivotCurrency", value)

    @property
    def pivot_index_name(self):
        """
        :return: str
        """
        return self._get_parameter("pivotIndexName")

    @pivot_index_name.setter
    def pivot_index_name(self, value):
        self._set_parameter("pivotIndexName", value)

    @property
    def quoted_currency(self):
        """
        :return: str
        """
        return self._get_parameter("quotedCurrency")

    @quoted_currency.setter
    def quoted_currency(self, value):
        self._set_parameter("quotedCurrency", value)

    @property
    def quoted_index_name(self):
        """
        :return: str
        """
        return self._get_parameter("quotedIndexName")

    @quoted_index_name.setter
    def quoted_index_name(self, value):
        self._set_parameter("quotedIndexName", value)



import numpy as np
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
from matplotlib import cm

from conftest import open_platform_session
from refinitiv.data.content import ipa


def convert_DateString_to_float(date_string_array):
    import datetime
    import matplotlib.dates as dates

    date_float_array = []
    for date_string in date_string_array:
        date_string = str(date_string)
        try:
            date_float = dates.date2num(
                datetime.datetime.strptime(date_string, "%Y-%m-%d")
            )
            date_float_array.append(date_float)
        except ValueError:
            date_float = dates.date2num(
                datetime.datetime.strptime(date_string, "%Y-%m-%dT%H:%M:%SZ")
            )
            date_float_array.append(date_float)
    return date_float_array


open_platform_session()

definition = ipa.surface_eti.Definition(
    underlying_definition=ipa.surface_eti.EtiSurfaceDefinition(
        instrument_code="BNPP.PA@RIC"
    ),
    surface_parameters=ipa.surface_eti.EtiCalculationParams(
        price_side=ipa.surface_eti.PriceSide.MID,
        volatility_model=ipa.surface_eti.VolatilityModel.SVI,
        x_axis=ipa.surface_eti.Axis.STRIKE,
        y_axis=ipa.surface_eti.Axis.DATE,
    ),
    surface_tag="1_MATRIX",
    surface_layout=ipa.surface_eti.SurfaceLayout(
        format=ipa.surface_eti.Format.MATRIX, y_point_count=10
    ),
)

res = definition.get_data()

definition = ipa.surfaces.Definition(
    universe=[
        ipa.surface_eti.Definition(
            underlying_definition=ipa.surface_eti.EtiSurfaceDefinition(
                instrument_code="BNPP.PA@RIC"
            ),
            surface_parameters=ipa.surface_eti.EtiCalculationParams(
                price_side=ipa.surface_eti.PriceSide.MID,
                volatility_model=ipa.surface_eti.VolatilityModel.SVI,
                x_axis=ipa.surface_eti.Axis.STRIKE,
                y_axis=ipa.surface_eti.Axis.DATE,
            ),
            surface_tag="1_MATRIX",
            surface_layout=ipa.surface_eti.SurfaceLayout(
                format=ipa.surface_eti.Format.MATRIX, y_point_count=10
            ),
        ),
        ipa.surface_eti.Definition(
            underlying_definition=ipa.surface_eti.EtiSurfaceDefinition(
                instrument_code="BNPP.PA@RIC"
            ),
            surface_parameters=ipa.surface_eti.EtiCalculationParams(
                price_side=ipa.surface_eti.PriceSide.MID,
                volatility_model=ipa.surface_eti.VolatilityModel.SVI,
                x_axis=ipa.surface_eti.Axis.DATE,
                y_axis=ipa.surface_eti.Axis.STRIKE,
            ),
            surface_tag="1_1",
            surface_layout=ipa.surface_eti.SurfaceLayout(
                format=ipa.surface_eti.Format.MATRIX, y_point_count=10
            ),
        ),
    ],
)
response = definition.get_data()
data = response.data
df = response.data.df
print(df)

surface = data.surfaces[0]

# VALUE ON AXIS
curve = surface.get_curve("25.35", ipa.surface_eti.Axis.X)
print("'25.35', X")
print(curve)

curve = surface.get_curve("2021-08-20", ipa.surface_eti.Axis.Y)
print("'2021-08-20', Y")
print(curve)

curve = surface.get_curve(53.1528001887883, ipa.surface_eti.Axis.Z)
print("53.1528001887883, Z")
print(curve)

point = surface.get_point(25.35, "2020-08-20")
print("'2020-08-20', 25.35")
print(point)

# VALUE NOT ON AXIS

curve = surface.get_curve("2021-08-25", ipa.surface_eti.Axis.Y)
print("'2021-08-25', Y")
print(curve)

curve = surface.get_curve("2021-08-30", ipa.surface_eti.Axis.Y)
print("'2021-08-30', Y")
print(curve)

curve = surface.get_curve("2021-09-15", ipa.surface_eti.Axis.Y)
print("'2021-09-15', Y")
print(curve)

curve = surface.get_curve(41, ipa.surface_eti.Axis.X)
print("41, X")
print(curve)

point = surface.get_point(26.45, "2020-08-25")
print("'2020-08-25', 26.45")
print(point)

surface = data.surfaces[1]

X = surface.x.astype(float)
Y, X = np.meshgrid(surface.y, surface.x)

ax = plt.axes(projection="3d")
ax.set_xlabel("moneyness")
ax.set_ylabel("time to expiry")
ax.set_zlabel("volatilities")

surf = ax.plot_surface(X, Y, Z, cmap=cm.coolwarm, linewidth=0, antialiased=False)
plt.show()



from .._content_data import Data
from ..._content_type import ContentType
from ..._tools import create_repr, validate_bool_value
from ...delivery._data._data_provider import DataProviderLayer, BaseResponse


class Definition(DataProviderLayer[BaseResponse[Data]]):
    """
    This class describe parameters to retrieve data for ESG universe.

    Parameters
    ----------
    use_field_names_in_headers: bool, optional
        Return field name as column headers for data instead of title

    Examples
    --------
    >>> from refinitiv.data.content import esg
    >>> definition = esg.universe.Definition()
    >>> response = definition.get_data()

    >>> response = await definition.get_data_async()
    """

    def __init__(
        self,
        use_field_names_in_headers: bool = False,
    ):
        validate_bool_value(use_field_names_in_headers)

        super().__init__(
            ContentType.ESG_UNIVERSE,
            use_field_names_in_headers=use_field_names_in_headers,
        )

    def __repr__(self):
        return create_repr(
            self,
            middle_path="universe",
            content=f"{{closure='{self._kwargs.get('closure')}'}}",
        )



__all__ = (
    "breakdown",
    "concentration",
    "investors",
    "recent_activity",
    "shareholders_history_report",
    "shareholders_report",
    "top_n_concentration",
)

from . import (
    breakdown,
    concentration,
    investors,
    recent_activity,
    shareholders_history_report,
    shareholders_report,
    top_n_concentration,
)



import allure
import pytest

from refinitiv.data.content.ownership.fund import investors
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
    get_expected_column,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
    check_extended_params_were_sent_in_request,
    check_dataframe_column_date_for_datetime_type,
)

EXPECTED_COLUMN_NAMES = [
    "instrument",
    "investorid",
    "investorpermid",
    "TR.FundPortfolioName",
    "TR.FundClassId",
    "TR.FundHoldingType",
    "TR.FdAdjPctOfShrsOutHeld",
    "TR.FundAdjShrsHeld",
    "TR.FdAdjShrsHeldChange",
    "TR.FundHoldingsDate",
    "TR.FundTurnoverRating",
]

EXPECTED_COLUMN_TITLES = [
    "Instrument",
    "Investor Id",
    "Investor Perm Id",
    "Investor Name",
    "Fund ClassId",
    "Investor Type",
    "% SharesOutstanding",
    "SharesHeld",
    "SharesHeld Change",
    "Holdings Date",
    "Turnover Rating",
]


@allure.suite("Content object - Ownership Fund Investors")
@allure.feature("Content object - Ownership Fund Investors")
@allure.severity(allure.severity_level.CRITICAL)
class TestFundInvestors:
    @allure.title(
        "Create Fund Investors definition object with valid params and get data"
    )
    @pytest.mark.caseid("37478871")
    @pytest.mark.parametrize(
        "universe,limit,use_field_names_in_headers,extended_params",
        [
            ("US02079K1079", 1, True, {"test_param": "test_value"}),
            (["TRI.N", "ONEX.CCP", "AAPL.O"], 20, False, None),
        ],
    )
    def test_create_fund_investors_object_with_valid_fields_and_get_data(
        self,
        open_platform_session,
        universe,
        limit,
        use_field_names_in_headers,
        extended_params,
    ):
        expected_column = get_expected_column(
            use_field_names_in_headers, EXPECTED_COLUMN_TITLES, EXPECTED_COLUMN_NAMES
        )
        response = investors.Definition(
            universe, limit, use_field_names_in_headers, extended_params
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )
        check_dataframe_column_date_for_datetime_type(response)
        check_extended_params_were_sent_in_request(response, extended_params)

    @allure.title(
        "Create Fund Investors definition object with mix valid and invalid params and get data asynchronously"
    )
    @pytest.mark.caseid("37478872")
    @pytest.mark.parametrize(
        "universe,invalid_universe,limit",
        [
            (["ONEX.CCP", "invalid universe"], "invalid universe", 150),
        ],
    )
    @pytest.mark.smoke
    async def test_create_fund_investors_object_with_mix_valid_and_invalid_params_and_get_data_async(
        self, open_platform_session_async, universe, invalid_universe, limit
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            investors.Definition(
                universe, use_field_names_in_headers=True, limit=limit
            ),
            investors.Definition(
                invalid_universe, use_field_names_in_headers=True, limit=limit
            ),
        )
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=None,
            expected_error_message="Failed to resolve identifiers ['invalid universe']",
        )
        check_non_empty_response_data(valid_response)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED_TWELVE,
            expected_error_message="Unable to resolve all requested identifiers. Requested items: ['invalid universe']",
        )
        check_response_dataframe_contains_columns_names(
            valid_response, EXPECTED_COLUMN_NAMES
        )
        check_response_dataframe_instrument_contains_expected_universes(
            valid_response, universe
        )



from typing import Optional, Union

import pandas as pd

from ..._types import OptDateTime
from ...content.news import story as _story
from ...content.news import headlines as _headlines
from ...content.news._sort_order import SortOrder


from enum import Enum, unique


@unique
class Format(Enum):
    TEXT = "Text"
    HTML = "Html"


def get_story(
    story_id: str,
    format: Optional[Union[Format, str]] = Format.HTML,
) -> str:
    """
    This function describes parameters to retrieve data for news story.

    Parameters
    ----------
    story_id : str
        News Story ID.
    format : Format, optional
        Specifies the type of response. If parameter Format.TEXT
        return text string, otherwise returns HTML response

    Returns
    -------
    str
        Story html or text response

    Examples
    --------
    >>> import refinitiv.data as rd
    >>> response = rd.news.get_story("urn:newsml:reuters.com:20220713:nL1N2YU10J")
    """

    response = _story.Definition(story_id).get_data()

    if format == Format.HTML or format == "Html":
        response = (
            response.data.raw.get("story", {}).get("storyHtml")
            or response.data.story.content.html
        )
    else:
        response = response.data.story.content.text

    return response


def get_headlines(
    query: str,
    count: int = 10,
    start: "OptDateTime" = None,
    end: "OptDateTime" = None,
    order_by: SortOrder = SortOrder.new_to_old,
) -> pd.DataFrame:
    """
    This function describes parameters to retrieve data for news headlines.

    Parameters
    ----------
    query: str
        The user search query.
    count: int, optional
        Count to limit number of headlines. Min value is 0. Default: 10
    start: str or timedelta, optional
        Beginning of date range.
        String format is: '%Y-%m-%dT%H:%M:%S'. e.g. '2016-01-20T15:04:05'.
    end: str or timedelta, optional
        End of date range.
        String format is: '%Y-%m-%dT%H:%M:%S'. e.g. '2016-01-20T15:04:05'.
    order_by: SortOrder
        Value from SortOrder enum. Default: SortOrder.new_to_old

    Returns
    -------
    pd.DataFrame
        Headlines dataframe

    Examples
    --------
    >>> from datetime import timedelta
    >>> import refinitiv.data as rd
    >>> definition = rd.news.get_headlines(
    ...     "Refinitiv",
    ...     start="20.03.2021",
    ...     end=timedelta(days=-4),
    ...     count=3
    ... )
    """

    response = _headlines.Definition(
        query=query, count=count, date_from=start, date_to=end, sort_order=order_by
    ).get_data()

    response = response.data.df

    return response



from typing import Optional

from . import CrossCurrencyCurveDefinitionKeys
from ....._object_definition import ObjectDefinition


class GetRequest(ObjectDefinition):
    """
    Get a Commodity & Energy curve definition

    Parameters
    ----------
    curve_definition : CrossCurrencyCurveDefinitionKeys, optional

    """

    def __init__(
        self,
        curve_definition: Optional[CrossCurrencyCurveDefinitionKeys] = None,
    ) -> None:
        super().__init__()
        self.curve_definition = curve_definition

    @property
    def curve_definition(self):
        """
        :return: object CrossCurrencyCurveDefinitionKeys
        """
        return self._get_object_parameter(
            CrossCurrencyCurveDefinitionKeys, "curveDefinition"
        )

    @curve_definition.setter
    def curve_definition(self, value):
        self._set_object_parameter(
            CrossCurrencyCurveDefinitionKeys, "curveDefinition", value
        )



from typing import Optional, Iterable, Union

from ._enums import ForwardCurvesOutputs
from ._forward_curve_definition import ForwardCurveDefinition
from ._models import Step, Turn
from ._swap_zc_curve_definition import SwapZcCurveDefinition
from ._swap_zc_curve_parameters import SwapZcCurveParameters
from ..curves import forward_curves
from ...._types import Strings

CurveDefinition = Optional[SwapZcCurveDefinition]
CurveParameters = Optional[SwapZcCurveParameters]
ForwardCurveDefinitions = Union[
    ForwardCurveDefinition, Iterable[ForwardCurveDefinition]
]
OptForwardCurveDefinitions = Optional[ForwardCurveDefinitions]

Outputs = Union[Strings, Iterable[ForwardCurvesOutputs]]
Universe = Union[forward_curves.Definition, Iterable[forward_curves.Definition]]

Steps = Union[Iterable[Step]]
Turns = Union[Iterable[Turn]]



from typing import Dict


def generate_docstring(description: str, methods: Dict[str, dict]):
    doc = [
        description,
        "",
        "    Methods",
        "    -------",
    ]

    for method_name, method_desc in methods.items():
        doc.append(f"    {method_name}")

        for param, desc in method_desc["args"].items():
            doc.append(f"        {param}")
            if "description" in desc:
                doc.append(f"            {desc['description']}")
            if "default" in desc:
                doc.append(f"            default: {repr(desc['default'])}")
            doc.append("")

    return "\n".join(doc)



# coding: utf8

__all__ = ["DateMovingConvention"]

from enum import Enum, unique


@unique
class DateMovingConvention(Enum):
    """
    The method to adjust dates.

    The possible values are:
        - BbswModifiedFollowing - Adjusts dates according to the BBSW Modified Following convention.
        - EveryThirdWednesday - dates are adjusted to the next every third Wednesday.
        - ModifiedFollowing - Adjusts dates according to the Modified Following
            convention - next business day unless it goes
            to the next month, preceding is used in that case.
        - NextBusinessDay - move to following date when public holiday.
        - NoMoving - no moving convention.
        - PreviousBusinessDay - move to preceding date when public holiday.
    """

    BBSW_MODIFIED_FOLLOWING = "BbswModifiedFollowing"
    EVERY_THIRD_WEDNESDAY = "EveryThirdWednesday"
    MODIFIED_FOLLOWING = "ModifiedFollowing"
    NEXT_BUSINESS_DAY = "NextBusinessDay"
    NO_MOVING = "NoMoving"
    PREVIOUS_BUSINESS_DAY = "PreviousBusinessDay"



# coding: utf8
from .._object_definition import ObjectDefinition
from ._enums import DiscountingType


class IIrVolModelDefinition(ObjectDefinition):
    def __init__(self, instrument_code=None, discounting_type=None):
        super().__init__()
        self.instrument_code = instrument_code
        self.discounting_type = discounting_type

    @property
    def discounting_type(self):
        """
        the discounting type of the IR vol model: OisDiscounting, or BorDiscounting (default)
        :return: enum DiscountingType
        """
        return self._get_enum_parameter(DiscountingType, "discountingType")

    @discounting_type.setter
    def discounting_type(self, value):
        self._set_enum_parameter(DiscountingType, "discountingType", value)

    @property
    def instrument_code(self):
        """
        The currency of the stripped cap surface, vol cube, or interest rate vol model
        :return: str
        """
        return self._get_parameter("instrumentCode")

    @instrument_code.setter
    def instrument_code(self, value):
        self._set_parameter("instrumentCode", value)



__all__ = (
    "BusinessDayConvention",
    "CdsConvention",
    "DayCountBasis",
    "Definition",
    "Direction",
    "DocClause",
    "Frequency",
    "PremiumLegDefinition",
    "PricingParameters",
    "ProtectionLegDefinition",
    "Seniority",
    "StubRule",
)

from ._cds_pricing_parameters import PricingParameters
from ._definition import Definition
from ._enums import (
    BusinessDayConvention,
    BusinessDayConvention,
    CdsConvention,
    DayCountBasis,
    Direction,
    DocClause,
    Frequency,
    Seniority,
    StubRule,
)

from ._models import PremiumLegDefinition, ProtectionLegDefinition



import itertools
from dataclasses import dataclass, field
from typing import Optional

from ._endpoint_data import RequestMethod

_id_iterator = itertools.count()


@dataclass
class Request:
    url: str
    method: str = RequestMethod.GET
    headers: dict = field(default_factory=dict)
    data: Optional[dict] = None
    params: Optional[dict] = None
    json: Optional[dict] = None
    closure: Optional[str] = None
    auto_retry: bool = False
    timeout: Optional[int] = None
    id: int = field(init=False, default_factory=lambda: next(_id_iterator))



from typing import TYPE_CHECKING

from ...._curves._bond_curves._curves._request import RequestItem
from ......_content_type import ContentType
from ......_tools import create_repr
from ....._content_provider_layer import ContentProviderLayer

if TYPE_CHECKING:
    from ......_types import OptStr, ExtendedParams
    from ...._curves._bond_curves._types import (
        CurveDefinition,
        CurveParameters,
    )


class Definition(ContentProviderLayer):
    """
    Generates the Bond curves for the definitions provided.

    Parameters
    ----------
    curve_definition : CreditCurveDefinition, optional
        CreditCurveDefinition object.
    curve_parameters : CreditCurveParameters, optional
        CreditCurveParameters object.
    curve_tag : str, optional
        A user-defined string to identify the interest rate curve. it can be used to
        link output results to the curve definition. limited to 40 characters. only
        alphabetic, numeric and '- _.#=@' characters are supported.

    extended_params : dict, optional
        If necessary other parameters.

    Methods
    -------
    get_data(session=session, on_response=on_response)
        Returns a response to the data platform
    get_data_async(session=None, on_response=None, async_mode=None)
        Returns a response asynchronously to the data platform

    Examples
    --------
    >>> from refinitiv.data.content.ipa.curves._bond_curves import curves
    >>> definition = curves.Definition(
    ...     curve_definition=curves.CreditCurveDefinition(
    ...         reference_entity="0#EUGOVPBMK=R",
    ...         reference_entity_type=curves.ReferenceEntityType.CHAIN_RIC
    ...     ))
    >>> response = definition.get_data()

    Using get_data_async
    >>> import asyncio
    >>> task = definition.get_data_async()
    >>> response = asyncio.run(task)
    """

    def __init__(
        self,
        curve_definition: "CurveDefinition" = None,
        curve_parameters: "CurveParameters" = None,
        curve_tag: "OptStr" = None,
        extended_params: "ExtendedParams" = None,
    ):
        request_item = RequestItem(
            curve_definition=curve_definition,
            curve_parameters=curve_parameters,
            curve_tag=curve_tag,
        )
        super().__init__(
            content_type=ContentType.BOND_CURVE,
            universe=request_item,
            extended_params=extended_params,
        )

    def __repr__(self):
        return create_repr(self, middle_path="_bond_curves.curves")



from datetime import datetime, timedelta
from typing import List, Optional, Union, TYPE_CHECKING

from numpy import iterable

from ._is_working_day_data_provider import IsWorkingDay, IsWorkingDays
from .._base_request_items import DateBase
from ..._enums import HolidayOutputs
from ...._content_provider_layer import ContentUsageLoggerMixin
from ....._content_type import ContentType
from ....._tools import create_repr, try_copy_to_list
from .....delivery._data._data_provider import DataProviderLayer, BaseResponse

if TYPE_CHECKING:
    from ....._types import ExtendedParams, OptStr, OptDateTime, OptStrStrs


class IsWorkingDayRequestItem(DateBase):
    def __init__(self, tag, date, calendars, currencies, holiday_outputs):
        super().__init__(date)
        self.tag = tag
        self.currencies = currencies
        self.calendars = calendars
        self.holiday_outputs = holiday_outputs

    @property
    def tag(self):
        """
        :return: str
        """
        return self._get_parameter("tag")

    @tag.setter
    def tag(self, value):
        self._set_parameter("tag", value)

    @property
    def currencies(self):
        """
        :return: list
        """
        return self._get_parameter("currencies")

    @currencies.setter
    def currencies(self, value):
        self._set_parameter("currencies", value)

    @property
    def calendars(self):
        """
        :return: list
        """
        return self._get_parameter("calendars")

    @calendars.setter
    def calendars(self, value):
        self._set_parameter("calendars", value)

    @property
    def holiday_outputs(self):
        """
        :return: list
        """
        return self._get_list_of_enums(HolidayOutputs, "holidayOutputs")

    @holiday_outputs.setter
    def holiday_outputs(self, value):
        self._set_list_of_enums(HolidayOutputs, "holidayOutputs", value)


class Definition(
    ContentUsageLoggerMixin[BaseResponse[IsWorkingDay]],
    DataProviderLayer[BaseResponse[IsWorkingDay]],
):
    """
    Is working day definition object

    Parameters
    ----------
        date: str or datetime or timedelta
            Date to test.
        calendars: list of str, optional
            Calendars to use the date for working day or weekend.
            Optional if currencies is provided.
        currencies: list of str, optional
            Currencies to use the date for working day or weekend.
            Optional if calendars is provided.
        holiday_outputs : HolidayOutputs or list of str, optional
            In case if test date is holiday you may request additional information about the holiday.
            Possible options are: Date, Names, Calendars, Countries
        tag: str, optional
            Reference tag to map particular response in payload.
        extended_params : dict, optional
            If necessary other parameters.

    Methods
    -------
    get_data(session=None, on_response=None, **kwargs)
        Returns a response to the data platform
    get_data_async(session=None, on_response=None, **kwargs)
        Returns a response asynchronously to the data platform

    Examples
    --------
     >>> import datetime
     >>> from refinitiv.data.content.ipa.dates_and_calendars import is_working_day
     >>>
     >>> definition = is_working_day.Definition(
     ...   tag="my request",
     ...   date=datetime.timedelta(0),
     ...   currencies=["EUR"],
     ...   holiday_outputs=["Date", "Names", "Calendars", "Countries"]
     ... )
     >>> response = definition.get_data()

     Using get_data_async
     >>> import asyncio
     >>> task = definition.get_data_async()
     >>> response = asyncio.run(task)
    """

    _USAGE_CLS_NAME = "IPA.DatesAndCalendars.IsWorkingDayDefinition"

    def __init__(
        self,
        date: "OptDateTime" = None,
        currencies: "OptStrStrs" = None,
        calendars: "OptStrStrs" = None,
        holiday_outputs: Optional[Union[List[HolidayOutputs], List[str]]] = None,
        tag: "OptStr" = None,
        extended_params: "ExtendedParams" = None,
    ):
        currencies = try_copy_to_list(currencies)
        calendars = try_copy_to_list(calendars)
        holiday_outputs = try_copy_to_list(holiday_outputs)
        self.extended_params = extended_params

        self.request_item = IsWorkingDayRequestItem(
            tag=tag,
            date=date,
            calendars=calendars,
            currencies=currencies,
            holiday_outputs=holiday_outputs,
        )

        super().__init__(
            data_type=ContentType.DATES_AND_CALENDARS_IS_WORKING_DAY,
            universe=[self.request_item],
            extended_params=extended_params,
        )

    def __repr__(self):
        return create_repr(self)


DefnDefns = Union[List[Definition], Definition]


class Definitions(
    ContentUsageLoggerMixin[BaseResponse[IsWorkingDays]],
    DataProviderLayer[BaseResponse[IsWorkingDays]],
):
    """
    Is working day definitions object

    Parameters
    ----------
    universe: Definition or list of Definition objects
        List of initialized Definition objects to retrieve data.

    Methods
    -------
    get_data(session=None, on_response=None, **kwargs)
        Returns a response to the data platform
    get_data_async(session=None, on_response=None, **kwargs)
        Returns a response asynchronously to the data platform

    Examples
    --------
     >>> import datetime
     >>> from refinitiv.data.content.ipa.dates_and_calendars import is_working_day
     >>>
     >>> first_definition = is_working_day.Definition(
     ...   tag="my request",
     ...   date=datetime.timedelta(0),
     ...   currencies=["EUR"],
     ...   holiday_outputs=["Date", "Names", "Calendars", "Countries"]
     ... )
     ...
     >>>
     >>> second_definition = is_working_day.Definition(
     ...   tag="my second request",
     ...   date="2020-01-01",
     ...   currencies=["EUR"],
     ...   holiday_outputs=["Date", "Names", "Calendars", "Countries"]
     ... )
     >>> response = is_working_day.Definitions([first_definition, second_definition]).get_data()

    Using get_data_async
     >>> import asyncio
     >>> task = is_working_day.Definitions([first_definition, second_definition]).get_data_async()
     >>> response = asyncio.run(task)
    """

    _USAGE_CLS_NAME = "IPA.DatesAndCalendars.IsWorkingDayDefinitions"

    def __init__(self, universe: "DefnDefns"):
        universe = try_copy_to_list(universe)
        if not iterable(universe):
            universe = [universe]

        request_items = []
        extended_params = []
        for item in universe:
            request_items.append(item.request_item)
            extended_params.append(item.extended_params)

        super().__init__(
            data_type=ContentType.DATES_AND_CALENDARS_IS_WORKING_DAY,
            universe=request_items,
            extended_params=extended_params,
        )

    def __repr__(self):
        return create_repr(self)



import abc
import copy
from dataclasses import dataclass, asdict
from typing import List, Union, Optional, Generic, TypeVar

import numpy as np

from refinitiv.data._tools import (
    EnumArgsParser,
    make_parse_enum,
    make_convert_to_enum,
    custom_inst_datetime_adapter,
)
from refinitiv.data._types import OptDateTime
from refinitiv.data.content.custom_instruments._enums import (
    VolumeBasedRolloverMethod,
    DayBasedRolloverMethod,
    SpreadAdjustmentMethod,
)

volume_based_rollover_method_enum_arg_parser = EnumArgsParser(
    parse=make_parse_enum(VolumeBasedRolloverMethod),
    parse_to_enum=make_convert_to_enum(VolumeBasedRolloverMethod),
)

day_based_rollover_method_enum_arg_parser = EnumArgsParser(
    parse=make_parse_enum(DayBasedRolloverMethod),
    parse_to_enum=make_convert_to_enum(DayBasedRolloverMethod),
)

spread_adjustment_method_enum_arg_parser = EnumArgsParser(
    parse=make_parse_enum(SpreadAdjustmentMethod),
    parse_to_enum=make_convert_to_enum(SpreadAdjustmentMethod),
)

_camel_to_snake = {
    "numberOfYears": "number_of_years",
    "startMonth": "start_month",
    "includeAllMonths": "include_all_months",
    "includeMonths": "include_months",
    "numberOfDays": "number_of_days",
    "monthsPrior": "months_prior",
    "joinAtDay": "join_at_day",
    "rollOccursWithinMonths": "roll_occurs_within_months",
    "rollOnExpiry": "roll_on_expiry",
    "startDate": "start_date",
    "spreadAdjustment": "spread_adjustment",
    "normalizeByWeight": "normalize_by_weight",
}


def convert_camel_to_snake(data: dict) -> dict:
    return {_camel_to_snake.get(name, name): value for name, value in data.items()}


T = TypeVar("T")


class Serializable(abc.ABC, Generic[T]):
    @abc.abstractmethod
    def _to_dict(self) -> dict:
        # for override
        pass

    @classmethod
    def _from_dict(cls, data: dict) -> T:
        # for override
        pass


@dataclass
class Constituent(Serializable["Constituent"]):
    """
    ric : str
        The instrument being considered.
    weight : int
        A numerical value multiplying the value of the default pricing field of ric.
    """

    ric: str
    weight: int

    def _to_dict(self) -> dict:
        return asdict(self)

    @classmethod
    def _from_dict(cls, data: dict) -> "Constituent":
        return cls(**data)


@dataclass
class Basket(Serializable["Basket"]):
    """
    constituents : List
        A list of objects, one for each row.
    normalize_by_weight : bool, optional
        True: means the total value computed above is divided by the total
              weight - to normalize to a total weight of 1.0.
        False: means that the basket prices are multiplied by weights and summed as is.
    """

    constituents: List[Union[Constituent, dict]]
    normalize_by_weight: bool = False

    def _to_dict(self) -> dict:
        return {
            "constituents": [
                item._to_dict() if isinstance(item, Constituent) else item
                for item in self.constituents
            ],
            "normalizeByWeight": self.normalize_by_weight,
        }

    @classmethod
    def _from_dict(cls, data: dict) -> "Basket":
        constituents = [
            Constituent._from_dict(constituent) for constituent in data["constituents"]
        ]
        normalize_by_weight = data["normalizeByWeight"]
        return cls(constituents=constituents, normalize_by_weight=normalize_by_weight)


@dataclass
class Months(Serializable["Months"]):
    """
    number_of_years: int
        Maximum of 30 - more results in an error
    include_all_month: bool
        Include all available months as a contract method
    include_months: List, Optional
        Include only selected months as contract method
    start_month: int, Optional

    """

    number_of_years: int
    include_all_months: Optional[bool] = None
    start_month: Optional[int] = None
    include_months: Optional[list] = None

    def _to_dict(self) -> dict:
        retval = {
            "numberOfYears": self.number_of_years,
        }
        if self.include_all_months:
            retval["includeAllMonths"] = self.include_all_months
        if self.start_month:
            retval["startMonth"] = self.start_month
        if self.include_months:
            retval["includeMonths"] = self.include_months
        return retval

    @classmethod
    def _from_dict(cls, data: dict) -> Optional["Months"]:
        obj = None
        months = data.get("months")
        if months:
            obj = cls(**convert_camel_to_snake(months))
        return obj


@dataclass
class VolumeBasedRollover(Serializable["VolumeBasedRollover"]):
    """
    “Method” for chaining individual contracts to produce UDCs.
        method:
        number_of_days: int
            The default for this is set at “1” which means the roll will occur between each pair of eligible contracts
            on the first day where the traded volume of the second contract exceeds that of the first.
        join_at_day: int
            Default is set at “1”. This can’t be altered when the “Number of Days” parameter is also set as “1” –
            not logical. However, you can alter this to any setting within a range limited by the minimum of “1” to
            a maximum of whatever value the “Number of Days” parameter is entered as.
        roll_occurs_within_months: int
        roll_on_expiry: bool
            This setting should the roll conditions not be met you will still get the roll to the next contract occurring
            but this will now be implemented on the expiry day of the first contract. This obviously requires that
            there is an overlap of data between the contracts. If param is False - History will terminate at the
            first point of failure to roll.

    """

    method: Union[VolumeBasedRolloverMethod, str]
    roll_occurs_within_months: int
    roll_on_expiry: bool = True
    number_of_days: int = 1
    join_at_day: int = 1

    def _to_dict(self) -> dict:
        return {
            "volumeBased": {
                "method": volume_based_rollover_method_enum_arg_parser.get_str(
                    self.method
                ),
                "numberOfDays": self.number_of_days,
                "joinAtDay": self.join_at_day,
                "rollOccursWithinMonths": self.roll_occurs_within_months,
                "rollOnExpiry": self.roll_on_expiry,
            }
        }

    @classmethod
    def _from_dict(cls, data: dict) -> "VolumeBasedRollover":
        data = convert_camel_to_snake(data)
        data["method"] = volume_based_rollover_method_enum_arg_parser.get_enum(
            data.get("method")
        )
        return cls(**data)


@dataclass
class DayBasedRollover(Serializable["DayBasedRollover"]):
    """
    This method supplies an entirely different approach to creating a futures continuation contract.
    The roll determination points are based purely on time and have nothing to do with comparing Volume and/or Open Interest.
        method: DayBasedRolloverMethod, str
                "daysBeforeExpiry" - the roll to the second contract occurs “X” business days before the expiry of the first contract.
                "daysBeforeEndOfMonth" - counts back business days from the final business day of a specified month –
                the default is the expiry month of the first contract.
                "daysAfterBeginningOfMonth" - the second bullet point re contracts such as CL/HO/RB.
        number_of_days: int
        month_prior: int

    """

    method: Union[DayBasedRolloverMethod, str]
    number_of_days: int
    months_prior: Optional[int] = None

    def _to_dict(self) -> dict:
        retval = {
            "dayBased": {
                "method": day_based_rollover_method_enum_arg_parser.get_str(
                    self.method
                ),
                "numberOfDays": self.number_of_days,
            }
        }
        if self.months_prior:
            retval["dayBased"]["monthsPrior"] = self.months_prior

        return retval

    @classmethod
    def _from_dict(cls, data: dict) -> "DayBasedRollover":
        data = convert_camel_to_snake(data)
        data["method"] = day_based_rollover_method_enum_arg_parser.get_enum(
            data.get("method")
        )
        return cls(**data)


@dataclass
class ManualItem:
    """
    month: int
    year: int
    star_date: str or datetime or timedelta
    """

    month: int
    year: int
    start_date: "OptDateTime"

    def _to_dict(self) -> dict:
        retval = {"month": self.month, "year": self.year}
        if self.start_date is not None:
            retval["startDate"] = custom_inst_datetime_adapter.get_str(self.start_date)

        return retval

    @classmethod
    def _from_dict(cls, data: dict) -> "ManualItem":
        month = data.get("month")
        year = data.get("year")
        start_date = np.datetime64(data.get("startDate"))
        return cls(month=month, year=year, start_date=start_date)


class ManualRollover(Serializable["ManualRollover"]):
    """
    “Manual” allows you to construct a futures continuation piece by piece – a process I refer to as “Daisy Chaining”.
    All that’s required is that you define consecutive contracts with the days that they should be spliced together.
        manual_items: args: ManualItem
    """

    def __init__(self, *args: ManualItem):
        self.manual_items = args

    def _to_dict(self) -> dict:
        return {
            "manual": [
                item._to_dict() if isinstance(item, ManualItem) else item
                for item in self.manual_items
            ]
        }

    @classmethod
    def _from_dict(cls, data: List) -> "ManualRollover":
        data = [ManualItem._from_dict(item) for item in data]
        return cls(*data)


@dataclass
class SpreadAdjustment(Serializable["SpreadAdjustment"]):
    """
    adjustment: str, optional
        None: prices are not changed at all.
        arithmetic: the arithmetic difference in price is used to adjust via addition/subtraction.
        percentage: instead of adding/subtracting from the price, a percentage adjustment is made instead.
    method: str, SpreadAdjustmentMethod
        "close-to-close",
        "open-to-open",
        "close-to-open",
        "close-to-open-old-gap",
        "close-to-open-new-gap"
    backwards: bool
        True: After retrieving all of the required futures contracts data, working out the roll dates and the basis gap
        adjustments that apply, the software then applies these basis adjustments working from the most recent contract backwards.

    """

    adjustment: Optional[Union[str, None]]
    method: Union[SpreadAdjustmentMethod, str]
    backwards: bool = True

    def _to_dict(self) -> dict:
        retval = {
            "method": spread_adjustment_method_enum_arg_parser.get_str(self.method),
            "backwards": self.backwards,
        }
        if self.adjustment:
            retval["adjustment"] = self.adjustment
        return retval

    @classmethod
    def _from_dict(cls, data: dict) -> "SpreadAdjustment":
        data = data.get("spreadAdjustment")
        data["method"] = spread_adjustment_method_enum_arg_parser.get_enum(
            data.get("method")
        )
        return cls(**data)


_rollover_key_name_by_class = {
    "dayBased": DayBasedRollover,
    "volumeBased": VolumeBasedRollover,
    "manual": ManualRollover,
}


@dataclass
class UDC(Serializable["UDC"]):
    """
    root: str
        This is the uppercase letter (or letters) before the month and year of the futures contract RIC code.
    rollover: VolumeBasedRollover, DayBasedRollover, ManualRollover
    spread_adjustment: SpreadAdjustment
        When joining the prices of two contracts as rollover from one to the other,
        the price series have to be adjusted so that the prices are continuous.
    months: Months
    """

    root: str
    rollover: Union[VolumeBasedRollover, DayBasedRollover, ManualRollover]
    spread_adjustment: SpreadAdjustment
    months: Optional[Months] = None

    def _to_dict(self) -> dict:
        retval = {
            "root": self.root,
            "rollover": self.rollover._to_dict(),
            "spreadAdjustment": self.spread_adjustment._to_dict(),
        }

        if self.months:
            retval["months"] = self.months._to_dict()
        return retval

    @classmethod
    def _from_dict(cls, data: dict) -> "UDC":
        rollover_obj = None
        data = copy.deepcopy(data)
        _data_rollover = data["rollover"]

        for rollover_key, rollover_class in _rollover_key_name_by_class.items():
            if rollover_key in _data_rollover:
                rollover_obj = rollover_class._from_dict(_data_rollover[rollover_key])
                break

        if not rollover_obj:
            raise ValueError("Rollover is missing, please define rollover parameter")

        return cls(
            root=data.get("root"),
            rollover=rollover_obj,
            spread_adjustment=SpreadAdjustment._from_dict(data),
            months=Months._from_dict(data),
        )



import json
import os

import pytest

from refinitiv.data import _configure as configure
from tests.integration.conftest import remove_config


def pytest_addoption(parser):
    parser.addoption(
        "--unit", action="store_true", default=False, help="run unit tests"
    )
    parser.addoption(
        "--integrate", action="store_true", default=False, help="run integration tests"
    )


def pytest_configure(config):
    config.addinivalue_line("markers", "unit: mark test as unit test")
    config.addinivalue_line("markers", "integrate: mark test as integration test")


def pytest_collection_modifyitems(config, items):
    if config.getoption("--unit"):
        skip = pytest.mark.skip(reason="run only unit tests")
        for item in items:
            if "unit" not in item.keywords:
                item.add_marker(skip)

    if config.getoption("--integrate"):
        skip = pytest.mark.skip(reason="run only integration tests")
        for item in items:
            if "integrate" not in item.keywords:
                item.add_marker(skip)


USER_CONFIG_PATH = os.path.join(
    os.path.expanduser("~"), configure._default_config_file_name
)


def remove_user_config():
    remove_config(USER_CONFIG_PATH)


def pytest_runtest_teardown(item, nextitem):
    configure._dispose()


@pytest.fixture()
def user_config_path():
    return USER_CONFIG_PATH


@pytest.fixture(scope="function")
def write_user_config(user_config_path):
    def inner(s):
        if isinstance(s, dict):
            s = json.dumps(s)
        f = open(user_config_path, "w")
        f.write(s)
        f.close()
        return user_config_path

    yield inner

    configure._dispose()

    remove_user_config()


@pytest.fixture
def reload_config_on_teardown():
    yield
    configure.reload()


def open_session_and_get_base_url(session):
    session.open()
    base_url = session._base_url
    session.close()

    return base_url



# coding: utf8

from enum import Enum, unique


@unique
class FixingFrequency(Enum):
    DAILY = "Daily"
    BI_WEEKLY = "BiWeekly"
    WEEKLY = "Weekly"
    MONTHLY = "Monthly"
    QUATERLY = "Quaterly"
    SEMI_ANNUAL = "SemiAnnual"
    ANNUAL = "Annual"



import pytest
from unittest.mock import patch

from refinitiv.data._content_type import ContentType


@pytest.fixture(
    scope="function",
    params=[
        ContentType.FORWARD_CURVE,
        ContentType.ZC_CURVES,
        ContentType.ZC_CURVE_DEFINITIONS,
        ContentType.SURFACES,
        ContentType.CONTRACTS,
    ],
)
def content_type(request):
    return request.param


@pytest.fixture(autouse=True, scope="module")
def change_delays():
    with patch.multiple(
        "refinitiv.data.content.ipa._ipa_content_provider",
        DELAY_BETWEEN_TWO_GET_ASYNC_OPERATION=0,
        DELAY_BEFORE_FIRST_GET_ASYNC_OPERATION=0,
    ):
        yield



from enum import Enum, auto


class NoValue(Enum):
    def __repr__(self):
        return "<%s.%s>" % (self.__class__.__name__, self.name)


class FilterType(NoValue):
    STREAM = auto()
    REST = auto()
    SYNC = auto()
    ASYNC = auto()
    LAYER_ACCESS = auto()
    LAYER_CONTENT = auto()
    LAYER_DELIVERY = auto()
    INIT = auto()



__all__ = ("Definition", "GrantPassword")

from .._core.session.grant_password import GrantPassword


class Definition(object):
    """
    Platform session.
    Can be defined indirectly using the name of a session defined
    in the configuration file or directly by specifying the other Definition parameters.

    Parameters
    ----------
    name: str, default "default"
        Session name
    app_key: str, optional
        Application key
    grant: GrantPassword object, optional
        Grants objects containing the credentials used to authenticate the user
        (or the machine) when connecting to the data platform.
        Several kind of grant objects can be used GrantPassword is the most common one.
    signon_control: bool, default True
        Controls the exclusive sign-on behavior when the user account
        (or computer account) for this session is concurrently used by another
        application. When this parameter is set to True, opening this session
        automatically disconnects the other application. When this parameter is set to
        False, the opening of this session fails preserving the other application.
    deployed_platform_host: str, optional
        Host name (or IP) and port to be used to connect to Real-Time Distribution
        System.
    deployed_platform_username: str, optional
        DACS user name identifying the user when to connecting
        to a Real-Time Distribution System
    dacs_position: str, optional
        DACS position identifying the terminal when connecting to a Real-Time
        Distribution System.
    dacs_application_id: str, optional
        Must contain the user's Data Access Control System application ID.
        For more information, refer to the DACS documentation on my.refinitiv.com

    Raises
    ----------
    Exception
        If app-key is not found in the config file and in arguments.

    Examples
    --------
    >>> import refinitiv.data as rd
    >>> definition = rd.session.platform.Definition(name="custom-session-name")
    >>> platform_session = definition.get_session()
    """

    def __init__(
        self,
        name: str = "default",
        app_key: str = None,
        grant=None,
        signon_control: bool = True,
        deployed_platform_host: str = None,
        deployed_platform_username: str = None,
        dacs_position: str = None,
        dacs_application_id: str = None,
    ) -> None:
        from .._core.session._session_provider import (
            _make_platform_session_provider_by_arguments,
        )

        if not isinstance(name, str):
            raise ValueError("Invalid session name type, please provide string.")

        self._create_session = _make_platform_session_provider_by_arguments(
            session_name=name,
            app_key=app_key,
            signon_control=signon_control,
            deployed_platform_host=deployed_platform_host,
            deployed_platform_username=deployed_platform_username,
            dacs_position=dacs_position,
            dacs_application_id=dacs_application_id,
            grant=grant,
        )

    def get_session(self):
        """
        Creates and returns the session.

        Returns
        -------
        The platform session instance.
        """
        session = self._create_session()
        return session



import asyncio
from time import sleep

import allure
import pytest

from refinitiv.data import OpenState
from refinitiv.data.content import custom_instruments as ci
from refinitiv.data._content_type import ContentType
from refinitiv.data.delivery._stream import stream_cxn_cache
from tests.integration.content.custom_instruments.conftest import (
    check_stream_instrument,
    check_open_and_close_stream_state,
    check_stream_snapshot,
    check_close_and_open_stream_state,
    is_stream_id_equal,
)
from tests.integration.content.helpers import (
    add_callbacks_for_universe_stream,
    check_triggered_events,
    check_stream_data,
)
from tests.integration.helpers import (
    assert_error,
)


@allure.suite("Content object - Custom Instrument")
@allure.feature("Custom Instrument Stream")
@allure.severity(allure.severity_level.CRITICAL)
class TestCustomInstrumentStream:
    @allure.title(
        "Create CustomInstrument stream multi-request with callbacks without updates and check events"
    )
    @pytest.mark.caseid("C40492347")
    @pytest.mark.smoke
    @pytest.mark.parametrize(
        "expected_events,expected_fields",
        [
            (
                ["Refresh", "Refresh", "Complete"],
                ["TRDPRC_1", "TRADE_DATE", "SALTIM_NS"],
            )
        ],
    )
    def test_create_custom_instrument_multi_request_stream_with_callbacks(
        self,
        open_session_with_rdp_creds_for_ci,
        create_instrument,
        expected_events,
        expected_fields,
    ):
        symbol_01 = create_instrument()
        symbol_02 = create_instrument()
        triggered_events = []
        stream = ci.Definition(
            universe=[symbol_01, symbol_02],
            api="streaming.custom-instruments.endpoints.resource",
        ).get_stream()
        add_callbacks_for_universe_stream(stream, triggered_events)

        check_stream_data(stream, [symbol_01, symbol_02], expected_fields)
        check_triggered_events(triggered_events, expected_events)
        check_stream_instrument([symbol_01, symbol_02], stream)

    @allure.title(
        "Create several custom instruments streams with valid Symbol in names using the same session"
    )
    @pytest.mark.caseid("C40492351")
    @pytest.mark.parametrize(
        "expected_events",
        [
            (["Refresh", "Complete", "Update"]),
        ],
    )
    async def test_create_custom_instrument_streams_with_same_session(
        self, open_desktop_session_async, create_instrument, expected_events
    ):
        symbol_01 = create_instrument()
        symbol_02 = create_instrument()
        symbol_03 = create_instrument()

        triggered_events_01 = []
        stream_01 = ci.Definition(universe=symbol_01).get_stream()
        add_callbacks_for_universe_stream(stream_01, triggered_events_01)

        triggered_events_02 = []
        stream_02 = ci.Definition(universe=symbol_02).get_stream()
        add_callbacks_for_universe_stream(stream_02, triggered_events_02)

        triggered_events_03 = []
        stream_03 = ci.Definition(universe=symbol_03).get_stream()
        add_callbacks_for_universe_stream(stream_03, triggered_events_03)

        await asyncio.gather(
            stream_01.open_async(), stream_02.open_async(), stream_03.open_async()
        )
        sleep(2)
        check_close_and_open_stream_state(stream_01, stream_02, stream_03, wait_time=2)
        check_triggered_events(triggered_events_01, expected_events)
        check_triggered_events(triggered_events_02, expected_events)
        check_triggered_events(triggered_events_03, expected_events)
        check_stream_instrument(symbol_01, stream_01)
        check_stream_instrument(symbol_02, stream_02)
        check_stream_instrument(symbol_03, stream_03)

    @allure.title(
        "Create two custom instruments streams with valid Symbol in names using separate sessions"
    )
    @pytest.mark.caseid("C40492355")
    @pytest.mark.parametrize(
        "expected_events",
        [
            (["Refresh", "Complete", "Update"]),
        ],
    )
    async def test_create_custom_instruments_with_separate_sessions(
        self,
        expected_events,
        open_platform_session_with_rdp_creds_async,
        open_desktop_session_async,
        create_instrument,
    ):
        symbol_01 = create_instrument(
            session=open_platform_session_with_rdp_creds_async
        )
        symbol_02 = create_instrument(session=open_desktop_session_async)

        triggered_events_01 = []
        stream_01 = ci.Definition(universe=symbol_01).get_stream(
            session=open_platform_session_with_rdp_creds_async
        )
        add_callbacks_for_universe_stream(stream_01, triggered_events_01)

        triggered_events_02 = []
        stream_02 = ci.Definition(universe=symbol_02).get_stream(
            session=open_desktop_session_async
        )
        add_callbacks_for_universe_stream(stream_02, triggered_events_02)

        await asyncio.gather(stream_01.open_async(), stream_02.open_async())
        sleep(3)
        assert stream_01.open_state == OpenState.Opened
        assert stream_02.open_state == OpenState.Opened

        check_triggered_events(triggered_events_01, expected_events)
        check_triggered_events(triggered_events_02, expected_events)
        assert not is_stream_id_equal(stream_01, stream_02)
        stream_01.close()
        stream_02.close()

    @pytest.mark.caseid("C41030998")
    @allure.title("Create Custom Instrument stream without required parameters")
    def test_create_custom_instrument_stream_without_required_parameters(self):
        with pytest.raises(TypeError) as error:
            ci.Definition().get_stream()
        assert_error(error, "universe")

    @allure.title(
        "Create Custom Instrument stream with non-existed Symbol and get error event"
    )
    @pytest.mark.caseid("C41030999")
    @pytest.mark.parametrize(
        "name,expected_events,expected_msg",
        [
            (
                "W)INVALID",
                ["Error"],
                "Custom instrument symbol is invalid. Valid format example: S)INSTRUMENTSYMBOL.USER_UUID",
            )
        ],
    )
    def test_create_custom_instrument_stream_with_invalid_symbol(
        self, open_desktop_session, name, expected_events, expected_msg
    ):
        stream = ci.Definition(universe=name).get_stream()
        triggered_events = []
        add_callbacks_for_universe_stream(stream, triggered_events)
        check_open_and_close_stream_state(stream, wait_before_close=1)
        check_triggered_events(triggered_events, expected_events)
        # assert_stream_error(error_list, expected_msg, symbol)

    @pytest.mark.caseid("C40492362")
    @allure.title("Close Custom Instrument stream by closing session")
    def test_close_custom_instrument_stream_after_session_close(
        self, open_session_with_rdp_creds_for_ci, create_instrument
    ):
        symbol = create_instrument()
        session = open_session_with_rdp_creds_for_ci
        stream = ci.Definition(universe=symbol).get_stream()
        stream.open()
        session.close()
        assert stream.open_state == OpenState.Closed, "Stream is not in closed state"

        is_cnx_alive = stream_cxn_cache.is_cxn_alive(
            session, ContentType.STREAMING_CUSTOM_INSTRUMENTS
        )
        assert not is_cnx_alive, f"Connection is alive"
        session.open()

    @allure.title("Create Custom Instrument stream with get_snapshot")
    @pytest.mark.caseid("C41031000")
    @pytest.mark.parametrize(
        "expected_fields", [(["Instrument", "TRDPRC_1", "TRADE_DATE", "SALTIM_NS"])]
    )
    def test_create_custom_instrument_stream_and_get_snapshot(
        self, open_session, create_instrument, expected_fields
    ):
        symbol = create_instrument()
        stream = ci.Definition(universe=symbol).get_stream()
        stream.open(with_updates=False)
        check_stream_snapshot(
            stream=stream, universe=symbol, expected_fields=expected_fields
        )

    @allure.title(
        "Create CustomInstrument stream with universe and put to extended_params new universe and check if it exist"
    )
    @pytest.mark.caseid("C43998924")
    @pytest.mark.parametrize(
        "expected_fields",
        [(["TRDPRC_1", "TRADE_DATE", "SALTIM_NS"],)],
    )
    def test_create_custom_instrument_stream_with_universe_and_put_to_extended_params_new_universe(
        self,
        open_desktop_session,
        create_instrument,
        expected_fields,
    ):
        symbol = create_instrument()
        extended_symbol = create_instrument()

        stream = ci.Definition(
            universe=symbol, extended_params={"universe": extended_symbol}
        ).get_stream()

        check_stream_instrument(extended_symbol, stream)
        check_stream_data(stream, extended_symbol, expected_fields)

    @allure.title("Open and close CustomInstrument stream with context manager")
    @pytest.mark.caseid("C44023012")
    def test_open_custom_instrument_streaming_with_context_manager(
        self, create_instrument, open_session_with_rdp_creds_for_ci
    ):
        symbol = create_instrument(session=open_session_with_rdp_creds_for_ci)
        ci_stream = ci.Definition(universe=symbol).get_stream()
        with ci_stream:
            assert ci_stream._stream.is_opened, f"Stream is not Opened"
        assert not ci_stream._stream.is_opened, f"Stream is not Сlosed"

    @allure.title("Open and close CustomInstrument stream with async context manager")
    @pytest.mark.caseid("C44023011")
    async def test_open_custom_instrument_streaming_with_async_context_manager(
        self, create_instrument, open_desktop_session_async
    ):
        symbol = create_instrument(session=open_desktop_session_async)
        ci_stream = ci.Definition(universe=symbol).get_stream()
        async with ci_stream:
            assert ci_stream._stream.is_opened, f"Stream is not Opened"
        assert not ci_stream._stream.is_opened, f"Stream is not Сlosed"

    @pytest.mark.caseid("")
    @allure.title("Create Custom Instrument stream with invalid api parameter")
    def test_create_custom_instrument_stream_with_invalid_api(
        self, open_session_with_rdp_creds_for_ci, create_instrument
    ):
        with pytest.raises(ValueError) as error:
            stream = ci.Definition(
                universe=create_instrument(),
                api="streaming.custom-instruments.endpoints.invalid",
            ).get_stream()
            stream.open()
        assert (
            str(error.value)
            == "Not an existing path apis.streaming.custom-instruments.endpoints.invalid to url into config file"
        )



from refinitiv.data.eikon import StreamingPrices
from tests.unit.conftest import StubSession


def test_stream_class_name():
    # given
    expected_class_name = "PricingStream"

    session = StubSession(is_open=True)
    stream = StreamingPrices(universe=["EUR=", "GBP="], session=session)

    # when
    class_name = str(type(stream["EUR="]))
    class_name = class_name.rsplit(".", 1)[-1][:-2]

    # then
    assert class_name == expected_class_name



import abc
from typing import Union, List

from numpy import iterable

from .._ipa_content_provider import IPAContentProviderLayer
from ...._content_type import ContentType
from ...._tools import try_copy_to_list


class BaseSurfaceDefinition(IPAContentProviderLayer, abc.ABC):
    pass


DefnDefns = Union[BaseSurfaceDefinition, List[BaseSurfaceDefinition]]


class Definitions(IPAContentProviderLayer):
    def __init__(
        self,
        universe: "DefnDefns",
    ):
        universe = try_copy_to_list(universe)
        if not iterable(universe):
            universe = [universe]

        super().__init__(
            content_type=ContentType.SURFACES,
            universe=universe,
            __plural__=True,
        )



import allure
import pytest

from refinitiv.data.errors import RDError
from refinitiv.data.content import news
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.news.conftest import (
    check_news_headlines_date_for_datetime_type,
)
from tests.integration.helpers import (
    check_response_status,
    check_non_empty_response_data,
    get_async_response_from_definition,
    check_index_column_contains_dates,
)


@allure.suite("Content object - TopNews")
@allure.feature("Content object - TopNews")
@allure.severity(allure.severity_level.CRITICAL)
class TestTopNews:
    @allure.title("Create TopNews object with top_news_id and call get_data")
    @pytest.mark.caseid("C51290262")
    @pytest.mark.parametrize(
        "news_id,revision", [("urn:newsml:reuters.com:20020923:SPDOC_119827232002", 17)]
    )
    @pytest.mark.smoke
    def test_create_top_news_object_and_get_data(
        self, open_platform_session, news_id, revision
    ):
        response = news.top_news.Definition(
            top_news_id=news_id, revision_id=revision
        ).get_data()

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_index_column_contains_dates(response)
        check_news_headlines_date_for_datetime_type(response)

    @allure.title("Create TopNews object using closed session")
    @pytest.mark.caseid("C51290263")
    @pytest.mark.parametrize(
        "news_id", ["urn:newsml:reuters.com:20020923:SPDOC_119827232002"]
    )
    def test_create_top_news_object_when_session_is_not_opened(
        self, news_id, open_platform_session
    ):
        session = open_platform_session
        session.close()
        with pytest.raises(ValueError) as error:
            news.top_news.Definition(top_news_id=news_id).get_data()
        assert str(error.value) == "Session is not opened. Can't send any request"

    @allure.title("Create TopNews object and call get_data_async")
    @pytest.mark.caseid("C51290264")
    @pytest.mark.parametrize(
        "news_id", ["urn:newsml:reuters.com:20020923:SPDOC_119827232002"]
    )
    async def test_create_top_news_object_and_get_data_async(
        self, open_platform_session_async, news_id
    ):
        response = await get_async_response_from_definition(
            news.top_news.Definition(top_news_id=news_id)
        )
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)

    @allure.title("Create TopNews object with the invalid top_news_id")
    @pytest.mark.caseid("C51290265")
    @pytest.mark.parametrize("news_id", ["invalid"])
    def test_create_top_news_object_with_invalid_story_id(
        self, open_platform_session, news_id
    ):
        with pytest.raises(RDError) as error:
            news.top_news.Definition(top_news_id=news_id).get_data()
        assert (
            str(error.value)
            == f'Error code 400 | topNewsId "{news_id}" is malformatted.'
        )



import pytest

from refinitiv.data._core.session._session_cxn_type import SessionCxnType
from refinitiv.data.delivery._stream._stream_cxn_config_provider import (
    DeployedCxnConfigProvider,
)
from tests.unit.conftest import StubSession, StubConfig, StubResponse, args


def create_deployed_cxn_config_provider(rawconfig):
    from tests.unit.delivery.stream.raw_infos import data

    response = StubResponse(data)
    config = StubConfig(rawconfig)
    session = StubSession(
        config=config, response=response, session_cxn_type=SessionCxnType.DEPLOYED
    )
    provider = DeployedCxnConfigProvider()
    return provider, session


@pytest.mark.parametrize(
    "rawconfig,expected_url",
    [
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            },
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "//amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            },
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "//amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            },
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            },
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
    ],
)
def test_configuration_parsed_correct_without_deployed_host_in_config(
    rawconfig, expected_url
):
    provider, session = create_deployed_cxn_config_provider(rawconfig)

    # when
    config = provider.get_cfg(session, "head.endpoints.tail")

    # then
    assert config.url == expected_url


@pytest.mark.parametrize(
    "deployed_host,expected_url",
    [
        args(
            input="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input="10.3.177.158:15000",
            expected="ws://10.3.177.158:15000/WebSocket",
        ),
        args(
            input="//amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input="amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input="//amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
        args(
            input="amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
    ],
)
def test_configuration_parsed_correct_with_deployed_host(deployed_host, expected_url):
    # given
    session = StubSession(
        deployed_platform_host=deployed_host, session_cxn_type=SessionCxnType.DEPLOYED
    )
    provider = DeployedCxnConfigProvider()

    # when
    config = provider.get_cfg(session, "head.endpoints.tail")

    # then
    assert config.url == expected_url



from typing import TYPE_CHECKING, Union

from .._content_data import Data
from ..._tools import create_repr
from ...delivery._data._data_provider import DataProviderLayer, BaseResponse
from ..._content_type import ContentType

if TYPE_CHECKING:
    from ._views import Views


class Definition(DataProviderLayer[BaseResponse[Data]]):
    """
    This class describe parameters to retrieve data for search metadata.

    Parameters
    ----------

    view : Views
        picks a subset of the data universe to search against. see Views

    Examples
    --------
    >>> from refinitiv.data.content import search
    >>> definition = search.metadata.Definition(view=search.Views.PEOPLE)
    """

    def __init__(self, view: Union["Views", str]):
        self._view = view

        super().__init__(
            data_type=ContentType.DISCOVERY_METADATA,
            view=self._view,
        )

    def __repr__(self):
        return create_repr(
            self,
            middle_path="metadata",
            content=f"{{view='{self._view}'}}",
        )



from typing import Callable, Any, Dict, Union

import pandas as pd

from .._content_type import ContentType
from ._df_build_type import DFBuildType
from ._df_builder import (
    dfbuilder_udf,
    dfbuilder_rdp,
    default_build_df,
    build_empty_df,
    build_dates_calendars_holidays_df,
    build_dates_calendars_df,
    build_dates_calendars_date_schedule_df,
)
from ._historical_df_builder import (
    custom_insts_builder,
    historical_builder,
)
from .custom_instruments._custom_instruments_data_provider import (
    custom_instruments_build_df,
)
from .ipa._curves._curves_builder_df import (
    bond_curve_build_df,
    cross_currency_curves_curve_build_df,
)
from .ipa._curves._curves_data_provider import (
    forward_curve_build_df,
    zc_curve_definitions_build_df,
    zc_curves_build_df,
    cross_currency_curves_definitions_search_build_df,
)
from .ipa.financial_contracts._contracts_data_provider import (
    financial_contracts_build_df,
)
from .news._tools import (
    news_build_df_rdp,
    news_build_df_udf,
)
from .news.top_news.hierarchy._df_builder import news_top_hierarchy_build_df
from .news.top_news._df_builder import news_top_build_df
from .pricing._pricing_content_provider import pricing_build_df
from .pricing.chain._chains_data_provider import chains_build_df
from .search._data_provider import (
    discovery_lookup_build_df,
    discovery_metadata_build_df,
    discovery_search_build_df,
)
from ..delivery._data._data_type import DataType

content_type_by_build_type: Dict[
    Union[ContentType, DataType], Callable[[Any, Dict[str, Any]], pd.DataFrame]
] = {
    ContentType.CHAINS: chains_build_df,
    ContentType.CONTRACTS: financial_contracts_build_df,
    ContentType.CUSTOM_INSTRUMENTS_INSTRUMENTS: custom_instruments_build_df,
    ContentType.CUSTOM_INSTRUMENTS_SEARCH: custom_instruments_build_df,
    ContentType.DATA_GRID_RDP: {
        DFBuildType.INDEX: dfbuilder_rdp.build_index,
        DFBuildType.DATE_AS_INDEX: dfbuilder_rdp.build_date_as_index,
    },
    ContentType.DATA_GRID_UDF: {
        DFBuildType.INDEX: dfbuilder_udf.build_index,
        DFBuildType.DATE_AS_INDEX: dfbuilder_udf.build_date_as_index,
    },
    ContentType.DEFAULT: default_build_df,
    ContentType.DISCOVERY_LOOKUP: discovery_lookup_build_df,
    ContentType.DISCOVERY_METADATA: discovery_metadata_build_df,
    ContentType.DISCOVERY_SEARCH: discovery_search_build_df,
    ContentType.ESG_BASIC_OVERVIEW: dfbuilder_rdp.build_index,
    ContentType.ESG_FULL_MEASURES: dfbuilder_rdp.build_index,
    ContentType.ESG_FULL_SCORES: dfbuilder_rdp.build_index,
    ContentType.ESG_STANDARD_MEASURES: dfbuilder_rdp.build_index,
    ContentType.ESG_STANDARD_SCORES: dfbuilder_rdp.build_index,
    ContentType.ESG_UNIVERSE: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_ACTUALS_ANNUAL: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_ACTUALS_INTERIM: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_ACTUALS_KPI_ANNUAL: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_ACTUALS_KPI_INTERIM: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_ANNUAL: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_NON_PERIODIC_MEASURES: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_PERIODIC_MEASURES_ANNUAL: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_PERIODIC_MEASURES_INTERIM: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_HISTORICAL_SNAPSHOTS_RECOMMENDATIONS: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_INTERIM: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_KPI_ANNUAL: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_KPI_HISTORICAL_SNAPSHOTS_KPI: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_KPI_INTERIM: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_NON_PERIODIC_MEASURES: dfbuilder_rdp.build_index,
    ContentType.ESTIMATES_VIEW_SUMMARY_RECOMMENDATIONS: dfbuilder_rdp.build_index,
    ContentType.FILINGS_RETRIEVAL: dfbuilder_rdp.build_index,
    ContentType.FILINGS_SEARCH: dfbuilder_rdp.build_index,
    ContentType.BOND_CURVE: bond_curve_build_df,
    ContentType.FORWARD_CURVE: forward_curve_build_df,
    ContentType.CROSS_CURRENCY_CURVES_CURVES: cross_currency_curves_curve_build_df,
    ContentType.CROSS_CURRENCY_CURVES_DEFINITIONS_CREATE: build_empty_df,
    ContentType.CROSS_CURRENCY_CURVES_DEFINITIONS_DELETE: build_empty_df,
    ContentType.CROSS_CURRENCY_CURVES_DEFINITIONS_GET: build_empty_df,
    ContentType.CROSS_CURRENCY_CURVES_DEFINITIONS_UPDATE: build_empty_df,
    ContentType.CROSS_CURRENCY_CURVES_DEFINITIONS_SEARCH: cross_currency_curves_definitions_search_build_df,
    ContentType.CROSS_CURRENCY_CURVES_TRIANGULATE_DEFINITIONS_SEARCH: default_build_df,
    # df will be built while merge responses from endpoint ###########
    ContentType.CUSTOM_INSTRUMENTS_INTERDAY_SUMMARIES: custom_insts_builder,
    ContentType.CUSTOM_INSTRUMENTS_EVENTS: custom_insts_builder,
    ContentType.CUSTOM_INSTRUMENTS_INTRADAY_SUMMARIES: custom_insts_builder,
    ContentType.HISTORICAL_PRICING_EVENTS: historical_builder,
    ContentType.HISTORICAL_PRICING_INTERDAY_SUMMARIES: historical_builder,
    ContentType.HISTORICAL_PRICING_INTRADAY_SUMMARIES: historical_builder,
    ##################################################################
    ContentType.NEWS_HEADLINES_RDP: news_build_df_rdp,
    ContentType.NEWS_HEADLINES_UDF: news_build_df_udf,
    ContentType.NEWS_TOP_NEWS: news_top_hierarchy_build_df,
    ContentType.NEWS_TOP_NEWS_HEADLINES: news_top_build_df,
    ContentType.NEWS_STORY_RDP: build_empty_df,
    ContentType.NEWS_STORY_UDF: build_empty_df,
    ContentType.NEWS_IMAGES: build_empty_df,
    ContentType.OWNERSHIP_CONSOLIDATED_BREAKDOWN: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_CONSOLIDATED_CONCENTRATION: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_CONSOLIDATED_INVESTORS: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_CONSOLIDATED_RECENT_ACTIVITY: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_CONSOLIDATED_SHAREHOLDERS_HISTORY_REPORT: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_CONSOLIDATED_SHAREHOLDERS_REPORT: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_CONSOLIDATED_TOP_N_CONCENTRATION: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_BREAKDOWN: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_CONCENTRATION: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_HOLDINGS: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_INVESTORS: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_RECENT_ACTIVITY: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_SHAREHOLDERS_HISTORY_REPORT: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_SHAREHOLDERS_REPORT: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_FUND_TOP_N_CONCENTRATION: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_INSIDER_SHAREHOLDERS_REPORT: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_INSIDER_TRANSACTION_REPORT: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_INVESTOR_HOLDINGS: dfbuilder_rdp.build_index,
    ContentType.OWNERSHIP_ORG_INFO: dfbuilder_rdp.build_index,
    ContentType.PRICING: pricing_build_df,
    ContentType.SURFACES: default_build_df,
    ContentType.DATES_AND_CALENDARS_COUNT_PERIODS: default_build_df,
    ContentType.DATES_AND_CALENDARS_DATE_SCHEDULE: build_dates_calendars_date_schedule_df,
    ContentType.DATES_AND_CALENDARS_HOLIDAYS: build_dates_calendars_holidays_df,
    ContentType.DATES_AND_CALENDARS_ADD_PERIODS: build_dates_calendars_df,
    ContentType.DATES_AND_CALENDARS_IS_WORKING_DAY: build_dates_calendars_df,
    ContentType.ZC_CURVE_DEFINITIONS: zc_curve_definitions_build_df,
    ContentType.ZC_CURVES: zc_curves_build_df,
}


def get_dfbuilder(
    content_type: ContentType, dfbuild_type: DFBuildType = DFBuildType.INDEX
) -> Callable[[Any, Dict[str, Any]], pd.DataFrame]:
    builder_by_build_type = content_type_by_build_type.get(content_type)

    if not builder_by_build_type:
        raise ValueError(f"Cannot find mapping for content_type={content_type}")

    if hasattr(builder_by_build_type, "get"):
        builder = builder_by_build_type.get(dfbuild_type)

    else:
        builder = builder_by_build_type

    if not builder:
        raise ValueError(f"Cannot find mapping for dfbuild_type={dfbuild_type}")

    return builder



from typing import TYPE_CHECKING

from ...._object_definition import ObjectDefinition

if TYPE_CHECKING:
    from ......_types import OptStrings, OptStr


class FieldFormulaDescription(ObjectDefinition):
    """
    Creates the Cross Currency curve definition with the definition provided.

    Parameters
    ----------
    historical_fid_priority : string, optional
        The list of historical fid (field identifier) name used to get the market data.
    real_time_fid_priority : string, optional
        The list of real-time fid (field identifier) name used to get the market data.
    formula : str, optional
        The formula name used to adjust the market data value.
    """

    def __init__(
        self,
        historical_fid_priority: "OptStrings" = None,
        real_time_fid_priority: "OptStrings" = None,
        formula: "OptStr" = None,
    ) -> None:
        super().__init__()
        self.historical_fid_priority = historical_fid_priority
        self.real_time_fid_priority = real_time_fid_priority
        self.formula = formula

    @property
    def historical_fid_priority(self):
        """
        The list of historical fid (field identifier) name used to get the market data.
        :return: list string
        """
        return self._get_list_parameter(str, "historicalFidPriority")

    @historical_fid_priority.setter
    def historical_fid_priority(self, value):
        self._set_list_parameter(str, "historicalFidPriority", value)

    @property
    def real_time_fid_priority(self):
        """
        The list of real-time fid (field identifier) name used to get the market data.
        :return: list string
        """
        return self._get_list_parameter(str, "realTimeFidPriority")

    @real_time_fid_priority.setter
    def real_time_fid_priority(self, value):
        self._set_list_parameter(str, "realTimeFidPriority", value)

    @property
    def formula(self):
        """
        The formula name used to adjust the market data value.
        :return: str
        """
        return self._get_parameter("formula")

    @formula.setter
    def formula(self, value):
        self._set_parameter("formula", value)



from enum import Enum, unique


@unique
class TenorTypes(str, Enum):
    ODD = "Odd"
    LONG = "Long"
    IMM = "IMM"
    BEGINNING_OF_MONTH = "BeginningOfMonth"
    END_OF_MONTH = "EndOfMonth"

    def __str__(self) -> str:
        return str(self.value)



import abc
from typing import TYPE_CHECKING, Type, Union, Generic, TypeVar

from ._endpoint_data import EndpointData, ABCData
from ._response import ABCResponse

if TYPE_CHECKING:
    from ._parsed_data import ParsedData

DT = TypeVar("DT")


class ABCDataFactory(Generic[DT], abc.ABC):
    @abc.abstractmethod
    def create_data(
        self, parsed_data: "ParsedData", owner: ABCResponse, **kwargs
    ) -> DT:
        pass

    @abc.abstractmethod
    def create_data_success(
        self, parsed_data: "ParsedData", owner: ABCResponse, **kwargs
    ) -> DT:
        pass

    @abc.abstractmethod
    def create_data_fail(
        self, parsed_data: "ParsedData", owner: ABCResponse, **kwargs
    ) -> DT:
        pass


class DataFactoryMixin(ABCDataFactory[DT]):
    data_class: Type[DT]

    @staticmethod
    def get_raw(parsed_data: "ParsedData") -> Union[dict, list, str]:
        return parsed_data.content_data

    def create_data(
        self, parsed_data: "ParsedData", owner_: ABCResponse, **kwargs
    ) -> DT:
        if owner_.is_success:
            return self.create_data_success(parsed_data, owner_=owner_, **kwargs)
        else:
            return self.create_data_fail(parsed_data, owner_=owner_, **kwargs)

    def create_data_success(
        self, parsed_data: "ParsedData", owner_: ABCResponse, **kwargs
    ) -> DT:
        return self.data_class(raw=self.get_raw(parsed_data), owner_=owner_, **kwargs)

    def create_data_fail(
        self, parsed_data: "ParsedData", owner_: ABCResponse, **kwargs
    ) -> DT:
        return self.data_class(
            raw=parsed_data.content_data or {}, owner_=owner_, **kwargs
        )


class DataFactory(DataFactoryMixin[EndpointData]):
    def __init__(self, data_class: Type[ABCData] = None):
        self.data_class = data_class or EndpointData



# coding: utf8
from .._object_definition import ObjectDefinition


class FxVolatilitySurfaceDefinition(ObjectDefinition):
    def __init__(self, fx_cross_code=None):
        super().__init__()
        self.fx_cross_code = fx_cross_code

    @property
    def fx_cross_code(self):
        """
        The ISO code of the cross currency (e.g. 'EURCHF').
        Mandatory.
        :return: str
        """
        return self._get_parameter("fxCrossCode")

    @fx_cross_code.setter
    def fx_cross_code(self, value):
        self._set_parameter("fxCrossCode", value)



from typing import List, Callable, TYPE_CHECKING

from ._data_provider import (
    DataProvider,
    ContentValidator,
    ValidatorContainer,
    RequestFactory,
)
from ..._tools import cached_property

if TYPE_CHECKING:
    from ._parsed_data import ParsedData


class EndpointValidator(ContentValidator):
    @cached_property
    def validators(self) -> List[Callable[["ParsedData"], bool]]:
        return []


class EndpointRequestFactory(RequestFactory):
    def get_body_parameters(self, *args, **kwargs):
        return kwargs.get("body_parameters") or {}

    def get_query_parameters(self, *args, **kwargs):
        return kwargs.get("query_parameters") or []

    def extend_body_parameters(self, body_params, **kwargs):
        return body_params


endpoint_data_provider = DataProvider(
    validator=ValidatorContainer(content_validator=EndpointValidator()),
    request=EndpointRequestFactory(),
)



from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    FormulaParameterDescription,
)
from refinitiv.data.content.ipa._curves._enums import InstrumentType
from refinitiv.data.content.ipa._curves._cross_currency_curves._definitions import (
    BidAskFieldsFormulaDescription,
)


def test_formula_parameter_description():
    # given
    expected_dict = {
        "instrumentType": "Bond",
        "instrumentCode": "string",
        "fields": {},
        "name": "string",
    }

    # when
    testing_obj = FormulaParameterDescription(
        instrument_type=InstrumentType.BOND,
        instrument_code="string",
        fields=BidAskFieldsFormulaDescription(),
        name="string",
    )

    # then
    assert testing_obj.get_dict() == expected_dict

    assert testing_obj.instrument_type == InstrumentType.BOND
    assert testing_obj.instrument_code == "string"
    assert testing_obj.fields == BidAskFieldsFormulaDescription()
    assert testing_obj.name == "string"



from typing import Union, List, Dict

from ._context import RDPMixin
from ._hp_context import HPContext


class HPRDPContext(RDPMixin, HPContext):
    def prepare_headers(self, raw: Union[list, dict], *args) -> List[Dict]:
        headers = [
            {"name": "instrument", "title": "Instrument"},
            {"name": "date", "title": "Date"},
        ]

        if isinstance(raw, list):
            raw = raw[0]["headers"]

        elif isinstance(raw, dict):
            raw = raw["headers"]

        else:
            raw = []

        for item in raw:
            name = item["name"]

            if name.lower() in {"date", "instrument"}:
                continue

            headers.append({"name": name, "title": name})

        return headers



import pytest

import refinitiv.data as rd


def test_get_default(session):
    rd.session.set_default(session)

    assert rd.session.get_default() is session


def test_get_default_error(session):
    with pytest.raises(
        AttributeError,
        match="No default session created yet. Please create a session first!",
    ):
        rd.session.get_default()


def test_set_default(session):
    try:
        rd.session.set_default(session)
        assert rd.session.get_default() is session
    except Exception as e:
        assert False, e


def test_clear_default_session(session):
    rd.session.set_default(None)

    with pytest.raises(
        AttributeError,
        match="No default session created yet. Please create a session first!",
    ):
        rd.session.get_default()


def test_invalid_input_set_default_session():
    session = "invalid_input_type"

    with pytest.raises(TypeError, match="Invalid argument"):
        rd.session.set_default(session)



from datetime import datetime


def check_amount_of_objects_in_response_not_bigger_than(expected_amount, response):
    values_list = response.data.raw["value"]
    values_list_size = len(values_list)
    assert values_list_size <= expected_amount, (
        f"Retrieved number of objects {values_list_size} "
        f"is bigger than expected: {expected_amount}"
    )


def convert_string_to_date_format(date_string, date_format):
    return datetime.strptime(date_string, date_format)


def check_no_objects_in_response(response):
    objects = response.data.raw["value"]
    assert not objects, f"Non-empty response.data.raw received: {response.data.raw}"



import os
import random

import pandas as pd
import pytest

from refinitiv.data._content_type import ContentType
from refinitiv.data.content._df_builder_factory import get_dfbuilder
from refinitiv.data.content.filings._retrieval_data_provider import (
    Error,
    DownloadFileError,
    FilingsRequestFactory,
    DownloadAllFileResponse,
    DownloadFileResponse,
    FilingsFile,
    ListOfFile,
    FilingsData,
)
from tests.unit.conftest import StubSession


# ---------------------------------------------------------------------------
#   Test FilingRequestFactory
# ---------------------------------------------------------------------------


@pytest.mark.parametrize(
    ("input_value", "expected_url"),
    [
        ({}, ""),
        ({"filename": "test"}, "/{filename}"),
        ({"dcn": "test"}, "/search/{identifier}/{value}"),
        ({"doc_id": "test"}, "/search/{identifier}/{value}"),
        ({"filing_id": "test"}, "/search/{identifier}/{value}"),
    ],
)
def test_filing_request_factory_get_url(input_value, expected_url):
    # given
    prefix_url = "/data/filings/v1/retrieval"
    session = None

    # when
    url = FilingsRequestFactory().get_url(session, prefix_url, **input_value)

    # then
    assert url == f"{prefix_url}{expected_url}"


@pytest.mark.parametrize(
    ("input_value", "expected_path_parameters"),
    [
        ({}, {}),
        ({"filename": "filename_1"}, {"filename": "filename_1"}),
        (
            {"dcn": "dcn_1"},
            {"identifier": "dcn", "value": "dcn_1"},
        ),
        (
            {"doc_id": "doc_id_1"},
            {"identifier": "docid", "value": "doc_id_1"},
        ),
        (
            {"filing_id": "filing_id_1"},
            {"identifier": "filingid", "value": "filing_id_1"},
        ),
    ],
)
def test_filing_request_factory_get_path_parameters(
    input_value, expected_path_parameters
):
    # when
    session = StubSession()
    path_parameters = FilingsRequestFactory().get_path_parameters(
        session, **input_value
    )

    # then
    assert path_parameters == expected_path_parameters


def test_filing_request_factory_get_header_parameters():
    # given
    session = StubSession(is_open=True)
    # header is currently hard-coded in their document
    expected_headers = {
        "ClientID": "API_Playground",
        "X-Api-Key": "155d9dbf-f0ac-46d9-8b77-f7f6dcd238f8",
        "Accept": "*/*",
    }

    # when
    headers = FilingsRequestFactory().get_header_parameters(session)

    # then
    assert headers == expected_headers


# ---------------------------------------------------------------------------
#   Test FilingsFile
# ---------------------------------------------------------------------------


@pytest.mark.parametrize("attrib", (["filename", "signed_url", "mimetype"]))
def test_filings_file_attrib(attrib):
    # when
    filings_file = FilingsFile()

    # then
    assert hasattr(filings_file, attrib)


def test_filing_file_download(mock_server_success):
    # given
    filename = "name.txt"
    server_port = mock_server_success.server_port

    # when
    response = FilingsFile("name", f"http://127.0.0.1:{server_port}").download()
    obj_id = hex(id(response))

    # then
    assert os.path.exists(filename)

    with open(filename, "r") as f:
        content = f.read()
    assert content == "test text"

    os.remove(filename)


def test_filing_file_download_http_error(mock_server_error):
    # given
    server_port = mock_server_error.server_port

    # when
    f = FilingsFile("name", f"http://127.0.0.1:{server_port}")

    # then
    with pytest.raises(DownloadFileError) as err:
        f.download()
    assert err.value.code == 404
    assert err.value.message == "File not found"


def test_filing_file_download_file_location_error(mock_server_success):
    # given
    server_port = mock_server_success.server_port

    # when
    f = FilingsFile("name", f"http://127.0.0.1:{server_port}")

    # then
    with pytest.raises(DownloadFileError) as err:
        f.download(
            path=os.path.join(
                os.getcwd(),
                f"FILINGS_INVALID_PATH_{str(random.randint(0, 999999))}",
            )
        )

    assert "No such directory exists" in err.value.message


def test_filing_file_download_none_error(mock_server_error):
    # when
    f = FilingsFile("name")

    # then
    with pytest.raises(DownloadFileError) as err:
        f.download()

    assert "Invalid type for url" in err.value.message


@pytest.mark.asyncio
async def test_filing_file_download_async(mock_server_success):
    # given
    filename = "name.txt"
    server_port = mock_server_success.server_port

    def on_complete(resp):
        assert len(resp.errors) == 0

    # when
    await FilingsFile("name", f"http://127.0.0.1:{server_port}").download_async(
        callback=on_complete
    )

    # then
    assert os.path.exists(filename)

    with open(filename, "r") as f:
        content = f.read()
    assert content == "test text"

    os.remove(filename)


@pytest.mark.asyncio
async def test_filing_file_download_async_http_error(mock_server_error):
    # given
    server_port = mock_server_error.server_port

    def on_complete(resp):
        assert len(resp.errors) == 1

    # when
    filing = FilingsFile("name", f"http://127.0.0.1:{server_port}")
    response = await filing.download_async(callback=on_complete)

    # then
    assert len(response.errors) == 1


@pytest.mark.asyncio
async def test_filing_file_download_async_file_location_error(mock_server_success):
    # given
    server_port = mock_server_success.server_port

    def on_complete(resp):
        assert len(resp.errors) == 1

    # when
    response = await FilingsFile(
        "name", f"http://127.0.0.1:{server_port}"
    ).download_async(
        path=os.path.join(
            os.getcwd(),
            f"FILINGS_INVALID_PATH_{str(random.randint(0, 999999))}",
        ),
        callback=on_complete,
    )

    # then
    assert "No such directory exists" in response.errors[0].message


@pytest.mark.asyncio
async def test_filing_file_download_async_none_error(mock_server_error):
    # given
    filing = FilingsFile("name")

    # when
    response = await filing.download_async()

    # then
    assert "Invalid type for url" in response.errors[0].message


# ---------------------------------------------------------------------------
#   Test FilingData
# ---------------------------------------------------------------------------


def test_filings_data_df_none():
    # given
    df_builder = get_dfbuilder(ContentType.FILINGS_RETRIEVAL)

    # when
    df = FilingsData(raw={}, use_field=False, dfbuilder=df_builder).df

    assert df is None


@pytest.mark.parametrize(
    ("input_value", "expected_data"),
    [
        (
            {"signedUrl": "http://example/file_pdf"},
            [["file_pdf", "http://example/file_pdf", ""]],
        ),
        (
            {
                "file_pdf": {"signedUrl": "http://example/file_pdf", "mimeType": "pdf"},
                "file_txt": {"signedUrl": "http://example/file_txt", "mimeType": "txt"},
            },
            [
                ["file_pdf", "http://example/file_pdf", "pdf"],
                ["file_txt", "http://example/file_txt", "txt"],
            ],
        ),
    ],
)
def test_filings_data_df(input_value, expected_data):
    # given
    columns = ["Filename", "SignedURL", "MimeType"]
    expected_df = pd.DataFrame(data=expected_data, columns=columns)
    df_builder = get_dfbuilder(ContentType.FILINGS_RETRIEVAL)

    # when
    data = FilingsData(input_value, use_field=False, dfbuilder=df_builder)
    df = data.df

    # then
    assert df.compare(expected_df).empty


def test_filing_data_files():
    # given
    content = {"signedUrl": "http://example/file_pdf"}
    df_builder = get_dfbuilder(ContentType.FILINGS_RETRIEVAL)

    # when
    data = FilingsData(content, use_field=False, dfbuilder=df_builder).files

    # then
    f = data[0]
    assert type(f) == FilingsFile
    assert f.filename == "file_pdf"
    assert f.signed_url == "http://example/file_pdf"


def test_filings_data_files_when_df_is_none():
    # given
    df_builder = get_dfbuilder(ContentType.FILINGS_RETRIEVAL)

    # when
    data = FilingsData(None, use_field=False, dfbuilder=df_builder)

    # then
    assert len(data.files) == 0


def test_filings_download_file_response_errors():
    # given
    kwargs = {"error_code": [400, 401], "error_message": ["Error1", "Error2"]}

    # when
    response = DownloadFileResponse(**kwargs)

    # then
    assert response.errors == [Error(400, "Error1"), Error(401, "Error2")]


def test_filings_download_all_file_response():
    # given
    errors = []
    files = []

    # when
    response = DownloadAllFileResponse(files, errors)

    # then
    assert response.data.files == files
    assert response.errors == errors


def test_filings_list_of_file_download(mock_server_success):
    # given
    filename = "name.txt"
    server_port = mock_server_success.server_port

    # when
    list_of_file = ListOfFile()
    list_of_file.append(FilingsFile("name", f"http://127.0.0.1:{server_port}"))
    list_of_file.download()

    # then
    assert os.path.exists(filename)

    with open(filename, "r") as f:
        content = f.read()
    assert content == "test text"

    os.remove(filename)


@pytest.mark.asyncio
async def test_filings_list_of_file_download_async(mock_server_success):
    # given
    filename = "name.txt"
    server_port = mock_server_success.server_port

    def get_callback(resp):
        assert len(resp.errors) == 0

    # when
    list_of_file = ListOfFile()
    list_of_file.append(FilingsFile("name", f"http://127.0.0.1:{server_port}"))
    await list_of_file.download_async(callback=get_callback)

    # then
    assert os.path.exists(filename)

    with open(filename, "r") as f:
        content = f.read()
    assert content == "test text"

    os.remove(filename)


def test_filings_list_of_file_download_empty_files():
    # when
    list_of_file = ListOfFile()
    # then
    with pytest.raises(DownloadFileError) as err:
        list_of_file.download()
    assert err.value.message == "Cannot download any file. Files are empty."


@pytest.mark.asyncio
async def test_filings_list_of_file_download_async_empty_files():
    # given
    def get_callback(resp):
        assert resp.errors[0].message == "Cannot download any file. Files are empty."

    # when
    list_of_file = ListOfFile()

    # then
    response = await list_of_file.download_async(callback=get_callback)
    assert response.errors[0].message == "Cannot download any file. Files are empty."



import pytest
import requests

from refinitiv.data._core import session
from refinitiv.data._core.session import UpdateEvent
from refinitiv.data._core.session._default_session_manager import Wrapper
from refinitiv.data._core.session.access_token_updater import AccessTokenUpdater
from refinitiv.data._core.session.auth_manager import TokenInfo
from refinitiv.data._core.session.refresh_token_updater import RefreshTokenUpdater
from refinitiv.data._core.session.tools import Delays
from tests.unit.conftest import StubSession, StubResponse

codes = requests.codes


@pytest.fixture(scope="function")
def auth_manager(request):
    #
    expires_in = 3
    marker_expires_in = request.node.get_closest_marker("expires_in")
    if marker_expires_in is not None:
        expires_in = marker_expires_in.args[0]

    #
    status_code = 200
    marker_status_code = request.node.get_closest_marker("status_code")
    if marker_status_code is not None:
        status_code = marker_status_code.args[0]

    content_data = {
        "access_token": "access_token",
        "expires_in": expires_in,
        "scope": "scope",
        "refresh_token": "refresh_token",
        "token_type": "token_type",
    }
    response = StubResponse(content_data)
    response.status_code = status_code
    session_mock = StubSession(response=response)
    auto_reconnect = False
    am = session.AuthManager(session_mock, auto_reconnect)
    yield am
    am.dispose()


@pytest.mark.parametrize("auto_reconnect", [True, False])
def test_authorize_is_true_if_success_authorization(auth_manager, auto_reconnect):
    # given
    auth_manager._auto_reconnect = auto_reconnect

    # when
    authorized = auth_manager.authorize()

    # then
    assert authorized is True


@pytest.mark.parametrize("auto_reconnect", [True, False])
def test_is_authorized_return_true_if_success_authorization(
    auth_manager, auto_reconnect
):
    # given
    auth_manager._auto_reconnect = auto_reconnect

    # when
    auth_manager.authorize()

    # then
    assert auth_manager.is_authorized() is True


@pytest.mark.status_code(404)
@pytest.mark.parametrize("auto_reconnect", [False])
def test_authorize_return_false_if_fail(auth_manager, auto_reconnect):
    # given
    auth_manager._auto_reconnect = auto_reconnect

    # when
    authorized = auth_manager.authorize()

    # then
    assert authorized is False


@pytest.mark.status_code(404)
@pytest.mark.parametrize("auto_reconnect", [False])
def test_is_authorized_return_false_if_closed(auth_manager, auto_reconnect):
    # given
    auth_manager._auto_reconnect = auto_reconnect

    # when
    testing_value = auth_manager.is_authorized()

    # then
    assert testing_value is False


@pytest.mark.parametrize("auto_reconnect", [True, False])
def test_authorize_twice(auth_manager, auto_reconnect):
    # given
    auth_manager._auto_reconnect = auto_reconnect

    # when
    auth_manager.authorize()
    testing_value = auth_manager.authorize()

    # then
    assert testing_value is True


def test_refresh_token_updater_will_raise_error_if_delay_less_or_equal_then_0():
    # given
    updater = RefreshTokenUpdater(
        StubSession(), Wrapper(), 1, lambda *args, **kwargs: None
    )

    # then
    with pytest.raises(ValueError, match="Delay must be greater than 0"):
        # when
        updater.delay = 0


def test_access_token_updater_will_raise_error_if_delay_less_or_equal_then_0():
    # given
    updater = AccessTokenUpdater(StubSession(), 1, lambda *args, **kwargs: None)

    # then
    with pytest.raises(ValueError, match="Delay must be greater than 0"):
        # when
        updater.delay = 0


@pytest.mark.parametrize(
    "status_code",
    [codes.ok, codes.bad, codes.unauthorized, codes.forbidden, codes.not_found],
)
def test_access_token_updater_can_process_status_codes(status_code):
    handled = False

    def handler(*args, **kwargs):
        nonlocal handled
        handled = True
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = AccessTokenUpdater(StubSession(response=response), 0.001, handler)

    # when
    updater.start()

    # then
    assert handled is True


@pytest.mark.parametrize(
    "status_code",
    [
        codes.ok,
    ],
)
def test_access_token_updater_return_event_ACCESS_TOKEN_SUCCESS(status_code):
    event = None

    def handler(event_, message, json_content):
        nonlocal event
        event = event_
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = AccessTokenUpdater(StubSession(response=response), 0.001, handler)

    # when
    updater.start()

    # then
    assert event == UpdateEvent.ACCESS_TOKEN_SUCCESS


@pytest.mark.parametrize(
    "status_code",
    [
        codes.bad,
        codes.unauthorized,
        codes.forbidden,
    ],
)
def test_access_token_updater_return_event_ACCESS_TOKEN_UNAUTHORIZED(status_code):
    event = None

    def handler(event_, message, json_content):
        nonlocal event
        event = event_
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = AccessTokenUpdater(StubSession(response=response), 0.001, handler)

    # when
    updater.start()

    # then
    assert event == UpdateEvent.ACCESS_TOKEN_UNAUTHORIZED


@pytest.mark.parametrize("status_code", [codes.not_found])
def test_access_token_updater_return_event_ACCESS_TOKEN_FAILED(status_code):
    event = None

    def handler(event_, message, json_content):
        nonlocal event
        event = event_
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = AccessTokenUpdater(StubSession(response=response), 0.001, handler)

    # when
    updater.start()

    # then
    assert event == UpdateEvent.ACCESS_TOKEN_FAILED


@pytest.mark.parametrize(
    "status_code",
    [codes.ok, codes.bad, codes.unauthorized, codes.forbidden, codes.not_found],
)
def test_refresh_token_updater_can_process_status_codes(status_code):
    handled = False

    def handler(*args, **kwargs):
        nonlocal handled
        handled = True
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = RefreshTokenUpdater(
        StubSession(response=response),
        TokenInfo("", 1, {""}, ""),
        0.001,
        handler,
    )

    # when
    updater.start()

    # then
    assert handled is True


@pytest.mark.parametrize(
    "status_code",
    [
        codes.ok,
    ],
)
def test_refresh_token_updater_return_event_REFRESH_TOKEN_SUCCESS(status_code):
    event = None

    def handler(event_, message, json_content):
        nonlocal event
        event = event_
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = RefreshTokenUpdater(
        StubSession(response=response),
        TokenInfo("", 1, {""}, ""),
        0.001,
        handler,
    )

    # when
    updater.start()

    # then
    assert event == UpdateEvent.REFRESH_TOKEN_SUCCESS


@pytest.mark.parametrize(
    "status_code",
    [
        codes.bad,
        codes.unauthorized,
        codes.forbidden,
    ],
)
def test_refresh_token_updater_return_event_REFRESH_TOKEN_BAD(status_code):
    event = None

    def handler(event_, message, json_content):
        nonlocal event
        event = event_
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = RefreshTokenUpdater(
        StubSession(response=response),
        TokenInfo("", 1, {""}, ""),
        0.001,
        handler,
    )

    # when
    updater.start()

    # then
    assert event == UpdateEvent.REFRESH_TOKEN_BAD


@pytest.mark.parametrize("status_code", [codes.not_found])
def test_refresh_token_updater_return_event_REFRESH_TOKEN_FAILED(status_code):
    event = None

    def handler(event_, message, json_content):
        nonlocal event
        event = event_
        updater.stop()

    # given
    response = StubResponse(status_code=status_code)
    updater = RefreshTokenUpdater(
        StubSession(response=response),
        TokenInfo("", 1, {""}, ""),
        0.001,
        handler,
    )

    # when
    updater.start()

    # then
    assert event == UpdateEvent.REFRESH_TOKEN_FAILED


def test_delays_next():
    # given
    delays = Delays([1])

    # when
    delay = delays.next()

    # then
    assert delay == 1


def test_delays_next_twice():
    # given
    delays = Delays([1, 2])

    # when
    delay = delays.next()
    delay = delays.next()

    # then
    assert delay == 2


def test_delays_next_twice_same_result():
    # given
    delays = Delays([1])

    # when
    delay = delays.next()
    delay = delays.next()

    # then
    assert delay == 1


def test_delays_reset():
    # given
    delays = Delays([2, 1])

    # when
    delay = delays.next()
    delay = delays.next()
    delays.reset()
    delay = delays.next()

    # then
    assert delay == 2


def test_auth_manager_mandatory_parameters_for_access_token_updater():
    # given
    content_data = {
        "access_token": "access_token",
        "refresh_token": "refresh_token",
    }
    session_mock = StubSession()
    am = session.AuthManager(session_mock, False)

    try:
        # when
        am._access_token_update_handler(
            UpdateEvent.ACCESS_TOKEN_SUCCESS, "", content_data
        )
    except Exception as e:
        assert False, e
    else:
        assert True


def test_auth_manager_mandatory_parameters_for_refresh_token_updater():
    # given
    content_data = {
        "access_token": "access_token",
        "refresh_token": "refresh_token",
    }
    session_mock = StubSession()
    am = session.AuthManager(session_mock, False)

    try:
        # when
        am._refresh_token_update_handler(
            UpdateEvent.REFRESH_TOKEN_SUCCESS, "", content_data
        )
    except Exception as e:
        assert False, e
    else:
        assert True



from refinitiv.data.delivery._data._request import Request

try:
    from unittest.mock import MagicMock, AsyncMock
except ImportError:
    from mock.mock import AsyncMock
    from unittest.mock import MagicMock

import pytest

from refinitiv.data.delivery import endpoint_request
from refinitiv.data.delivery._data._endpoint_data import RequestMethod
from tests.unit.conftest import StubSession, StubResponse


def test_get_data():
    # given
    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(
        url="test_url",
        method="test_method",
        path_parameters={"path_parameters_key": "path_parameters_value"},
        query_parameters={"query_parameters_key": "query_parameters_value"},
        header_parameters={"header_parameters_key": "header_parameters_value"},
        body_parameters={"body_parameters_key": "body_parameters_value"},
    )

    # when
    response = definition.get_data(session)

    # then
    assert response is not None


def test_get_data_multiple_body_parameters():
    # given
    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(
        url="test_url",
        method="test_method",
        path_parameters={"path_parameters_key": "path_parameters_value"},
        query_parameters={"query_parameters_key": "query_parameters_value"},
        header_parameters={"header_parameters_key": "header_parameters_value"},
        body_parameters=[
            {"header_parameters_key": "header_parameters_value"},
            {"another_header_parameters_key": "another_header_parameters_value"},
        ],
    )

    # when
    response = definition.get_data(session)

    # then
    assert response is not None


def test_get_data_raise_error_if_no_session_passed_and_no_default_session():
    # given
    definition = endpoint_request.Definition(url="test_url")

    # then
    with pytest.raises(AttributeError) as error:
        # when
        definition.get_data()
    assert (
        error.value.args[0]
        == "No default session created yet. Please create a session first!"
    )


def test_get_data_simple():
    # given
    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(url="test_url")

    # when
    response = definition.get_data(session)

    # then
    assert response is not None


@pytest.mark.asyncio
async def test_get_data_async():
    # given
    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(url="test_url")

    # when
    response = await definition.get_data_async(session)

    # then
    assert response is not None


@pytest.mark.asyncio
async def test_get_data_async_raise_error_if_no_session_passed():
    # given
    definition = endpoint_request.Definition(url="test_url")

    # then
    with pytest.raises(AttributeError):
        # when
        await definition.get_data_async()


def test_get_data_raise_error_if_path_params_not_provided():
    definition = endpoint_request.Definition(
        url="/data/historical-pricing/v1/views/events/{universe}"
    )

    with pytest.raises(
        ValueError,
        match="Path parameter 'universe' is missing, please provide path parameter",
    ):
        definition.get_data()


def test_get_data_raise_error_if_url_empty():
    definition = endpoint_request.Definition(url="")

    with pytest.raises(
        ValueError, match="Requested URL is missing, please provide valid URL"
    ):
        definition.get_data()


@pytest.mark.parametrize(
    "property",
    [
        "url",
        "method",
        "path_parameters",
        "query_parameters",
        "header_parameters",
        "body_parameters",
    ],
)
def test_definition_has_properties(property):
    definition = endpoint_request.Definition("")

    assert hasattr(definition, property)


def test_changed_url_is_passing_to_get_data():
    # given
    expected_value = "expected_url"

    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(url="url")
    definition._provider = MagicMock()

    # when
    definition.url = expected_value

    # then
    definition.get_data(session)
    definition._provider.get_data.assert_called_once_with(
        session,
        expected_value,
        method=None,
        path_parameters=None,
        query_parameters=None,
        header_parameters=None,
        body_parameters=None,
    )


def test_changed_method_is_passing_to_get_data():
    # given
    expected_value = RequestMethod.POST

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, method=RequestMethod.GET)
    definition._provider = MagicMock()

    # when
    definition.method = expected_value

    # then
    definition.get_data(session)
    definition._provider.get_data.assert_called_once_with(
        session,
        url,
        method=expected_value,
        path_parameters=None,
        query_parameters=None,
        header_parameters=None,
        body_parameters=None,
    )


def test_changed_path_parameters_is_passing_to_get_data():
    # given
    expected_value = {"path_parameters_key": "path_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, path_parameters={})
    definition._provider = MagicMock()

    # when
    definition.path_parameters = expected_value

    # then
    definition.get_data(session)
    definition._provider.get_data.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=expected_value,
        query_parameters=None,
        header_parameters=None,
        body_parameters=None,
    )


def test_changed_query_parameters_is_passing_to_get_data():
    # given
    expected_value = {"query_parameters_key": "query_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, query_parameters={})
    definition._provider = MagicMock()

    # when
    definition.query_parameters = expected_value

    # then
    definition.get_data(session)
    definition._provider.get_data.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=None,
        query_parameters=expected_value,
        header_parameters=None,
        body_parameters=None,
    )


def test_changed_header_parameters_is_passing_to_get_data():
    # given
    expected_value = {"header_parameters_key": "header_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, header_parameters={})
    definition._provider = MagicMock()

    # when
    definition.header_parameters = expected_value

    # then
    definition.get_data(session)
    definition._provider.get_data.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=None,
        query_parameters=None,
        header_parameters=expected_value,
        body_parameters=None,
    )


def test_changed_body_parameters_is_passing_to_get_data():
    # given
    expected_value = {"body_parameters_key": "body_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, body_parameters={})
    definition._provider = MagicMock()

    # when
    definition.body_parameters = expected_value

    # then
    definition.get_data(session)
    definition._provider.get_data.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=None,
        query_parameters=None,
        header_parameters=None,
        body_parameters=expected_value,
    )


def test_error_is_a_string():
    # given
    expected_value = "BAD_REQUEST"
    response = StubResponse(
        {
            "requestId": "f4a1eae6-de2b-4946-8f59-00185a217145",
            "timestamp": "2022-05-19T08:53:43.631Z",
            "status": 400,
            "message": "Missing required parameter: type",
            "error": expected_value,
        },
        status_code=400,
    )
    session = StubSession(is_open=True, response=response)
    definition = endpoint_request.Definition(
        method=RequestMethod.POST,
        url="https://api.refinitiv.com/discovery/symbology/v1/lookup",
    )

    # when
    try:
        response = definition.get_data(session)
    except Exception as e:
        assert False, str(e)
    else:
        # then
        error = response.errors[0]
        assert error.message == expected_value


@pytest.mark.asyncio
async def test_changed_url_is_passing_to_get_data_async():
    # given
    expected_value = "expected_url"

    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(url="url")
    definition._provider = AsyncMock()

    # when
    definition.url = expected_value

    # then
    await definition.get_data_async(session)
    definition._provider.get_data_async.assert_called_once_with(
        session,
        expected_value,
        method=None,
        path_parameters=None,
        query_parameters=None,
        header_parameters=None,
        body_parameters=None,
    )


@pytest.mark.asyncio
async def test_changed_method_is_passing_to_get_data_async():
    # given
    expected_value = RequestMethod.POST

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, method=RequestMethod.GET)
    definition._provider = AsyncMock()

    # when
    definition.method = expected_value

    # then
    await definition.get_data_async(session)
    definition._provider.get_data_async.assert_called_once_with(
        session,
        url,
        method=expected_value,
        path_parameters=None,
        query_parameters=None,
        header_parameters=None,
        body_parameters=None,
    )


@pytest.mark.asyncio
async def test_changed_path_parameters_is_passing_to_get_data_async():
    # given
    expected_value = {"path_parameters_key": "path_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, path_parameters={})
    definition._provider = AsyncMock()

    # when
    definition.path_parameters = expected_value

    # then
    await definition.get_data_async(session)
    definition._provider.get_data_async.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=expected_value,
        query_parameters=None,
        header_parameters=None,
        body_parameters=None,
    )


@pytest.mark.asyncio
async def test_changed_query_parameters_is_passing_to_get_data_async():
    # given
    expected_value = {"query_parameters_key": "query_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, query_parameters={})
    definition._provider = AsyncMock()

    # when
    definition.query_parameters = expected_value

    # then
    await definition.get_data_async(session)
    definition._provider.get_data_async.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=None,
        query_parameters=expected_value,
        header_parameters=None,
        body_parameters=None,
    )


@pytest.mark.asyncio
async def test_changed_header_parameters_is_passing_to_get_data_async():
    # given
    expected_value = {"header_parameters_key": "header_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, header_parameters={})
    definition._provider = AsyncMock()

    # when
    definition.header_parameters = expected_value

    # then
    await definition.get_data_async(session)
    definition._provider.get_data_async.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=None,
        query_parameters=None,
        header_parameters=expected_value,
        body_parameters=None,
    )


@pytest.mark.asyncio
async def test_changed_body_parameters_is_passing_to_get_data_async():
    # given
    expected_value = {"body_parameters_key": "body_parameters_value"}

    session = StubSession(is_open=True)
    url = "url"
    definition = endpoint_request.Definition(url=url, body_parameters={})
    definition._provider = AsyncMock()

    # when
    definition.body_parameters = expected_value

    # then
    await definition.get_data_async(session)
    definition._provider.get_data_async.assert_called_once_with(
        session,
        url,
        method=None,
        path_parameters=None,
        query_parameters=None,
        header_parameters=None,
        body_parameters=expected_value,
    )


@pytest.mark.parametrize(
    "input_method, expected_method",
    [
        (RequestMethod.GET, "GET"),
        (RequestMethod.POST, "POST"),
        (RequestMethod.DELETE, "DELETE"),
        (RequestMethod.PUT, "PUT"),
        ("GET", "GET"),
        ("POST", "POST"),
        ("DELETE", "DELETE"),
        ("PUT", "PUT"),
        ("ANY_METHOD__", "ANY_METHOD__"),
    ],
)
def test_endpoint_request_definition_method_argument(input_method, expected_method):
    def assert_method(session):
        def inner(request: Request):
            # then
            assert request.method == expected_method
            return StubResponse()

        session.http_request = inner

    # given
    session = StubSession(is_open=True)
    definition = endpoint_request.Definition(
        url="test_url",
        method=input_method,
    )

    # when
    assert_method(session)
    definition.get_data(session)



from typing import Union

from ._file_downloader_facade import FileDownloader
from ._iter_object import CFSFile
from ._stream import CFSStream


class Definition:
    """
    Parameters
    __________
        file: dict, CFSFile
            dictionary with the keys 'id' and 'filename' or CFSFile object
            if 'filename' contains any of the characters ':/|\\?*<>"' these characters will be removed

    Methods
    _______
        retrieve(session)
            Returns FileDownloader object
    """

    def __init__(self, file: Union[dict, CFSFile]):
        self._file_id = file["id"]
        self._filename_ext = file["filename"]

    def retrieve(self, session=None) -> FileDownloader:
        stream = CFSStream(id=self._file_id).get_data(session=session)
        raw = stream.data.raw

        if raw.get("error", None):
            raise ValueError(raw["error"]["message"])

        url = raw.get("url", None)

        if not url:
            raise FileNotFoundError("file not found")

        return FileDownloader(url, self._filename_ext)



from enum import Enum, unique
from typing import Union, List, Optional

from ..._tools import make_enum_arg_parser_by_members


@unique
class SymbolTypes(Enum):
    """
    Symbol types to send in request, by default "RIC" is using.
    """

    RIC = "RIC"
    ISIN = "IssueISIN"
    CUSIP = "CUSIP"
    SEDOL = "SEDOL"
    TICKER_SYMBOL = "TickerSymbol"
    OA_PERM_ID = "IssuerOAPermID"
    LIPPER_ID = "FundClassLipperID"


SYMBOL_TYPE_VALUES = tuple(t.value for t in SymbolTypes)

OptSymbolTypes = Optional[Union[str, List[str], SymbolTypes, List[SymbolTypes]]]

symbol_types_arg_parser = make_enum_arg_parser_by_members(SymbolTypes)



# coding: utf8

__all__ = ["HolidayOutputs"]

from enum import Enum, unique
from typing import Union, List, Optional


@unique
class HolidayOutputs(Enum):
    """
    Method to request additional information about the holiday.

    The possible values are:
        - Date (to retrieve holiday date),
        - Names (to retrieve holiday names.
            Holiday names might be different.
            It depends on counties and calendars.),
        - Calendars (to retrieve calendars for which this holidays belong),
        - Countries  (to retrieve countries for which this holidays belong).
    """

    DATE = "Date"
    NAMES = "Names"
    CALENDARS = "Calendars"
    COUNTRIES = "Countries"


OptHolidayOutputs = Optional[
    Union[str, List[str], HolidayOutputs, List[HolidayOutputs]]
]



class SurfacePoint:
    def __init__(self, x, y, z) -> None:
        self.x = x
        self.y = y
        self.z = z

    def __str__(self) -> str:
        return f"x={self.x}, y={self.y}, z={self.z}"

    def __repr__(self):
        return f"SurfacePoint(x={self.x}, y={self.y}, z={self.z})"



from refinitiv.data.content.ipa.dates_and_calendars import count_periods
from refinitiv.data.content.ipa.dates_and_calendars.count_periods._count_periods_data_provider import (
    CountPeriodsResponseFactory,
)
from refinitiv.data.delivery._data._data_provider import ParsedData


MOCKED_COUNT_PERIODS_DATA = ParsedData(
    {"http_status_code": 200, "http_reason": "OK"},
    {},
    [{"count": 6.0, "tag": "first tag", "tenor": "WD"}],
)

MOCKED_MULTIPLE_COUNT_PERIODS_DATA = ParsedData(
    {"http_status_code": 200, "http_reason": "OK"},
    {},
    [
        {"count": 6.0, "tag": "first tag", "tenor": "WD"},
        {"count": 9.0, "tag": "second tag", "tenor": "WD"},
    ],
)


def test_date_schedule():
    definition = count_periods.Definition(
        tag="first tag",
        start_date="2019-04-30",
        end_date="2020-04-30",
        period_type=count_periods.PeriodType.WORKING_DAY,
        currencies=["EUR"],
        calendars=["EMU"],
    )

    assert definition.request_item.tag == "first tag"
    assert definition.request_item.start_date == "2019-04-30"
    assert definition.request_item.end_date == "2020-04-30"
    assert definition.request_item.calendars == ["EMU"]
    assert definition.request_item.currencies == ["EUR"]
    assert definition.request_item.period_type == count_periods.PeriodType.WORKING_DAY


def test_holidays_create_success():
    response_factory = CountPeriodsResponseFactory()
    response = response_factory.create_success(MOCKED_COUNT_PERIODS_DATA)
    assert not response.data.df.empty
    assert response.data.raw == MOCKED_COUNT_PERIODS_DATA.content_data
    assert response.data.counted_period.count == 6
    assert response.data.counted_period.tag == "first tag"
    assert response.data.counted_period.tenor == "WD"


def test_holidays_definitions_create_success():
    response_factory = CountPeriodsResponseFactory()
    response = response_factory.create_success(MOCKED_MULTIPLE_COUNT_PERIODS_DATA)
    assert not response.data.df.empty
    assert response.data.raw == MOCKED_MULTIPLE_COUNT_PERIODS_DATA.content_data
    assert response.data.counted_periods[0].count == 6
    assert response.data.counted_periods[0].tag == "first tag"
    assert response.data.counted_periods[0].tenor == "WD"
    assert response.data.counted_periods[1].count == 9
    assert response.data.counted_periods[1].tag == "second tag"
    assert response.data.counted_periods[1].tenor == "WD"



import abc


class UniverseExpander:
    @property
    @abc.abstractmethod
    def _universe(self):
        pass

    def __iter__(self):
        self._n = 0
        return self

    def __next__(self):
        if self._n < len(self._universe):
            result = self._universe[self._n]
            self._n += 1
            return result
        raise StopIteration



from enum import Enum, unique


@unique
class BusinessSector(Enum):
    ACADEMIC_AND_EDUCATIONAL_SERVICES = "AcademicAndEducationalServices"
    APPLIED_RESOURCES = "AppliedResources"
    AUTOMOBILES_AND_AUTO_PARTS = "AutomobilesAndAutoParts"
    BANKING_AND_INVESTMENT_SERVICES = "BankingAndInvestmentServices"
    CHEMICALS = "Chemicals"
    COLLECTIVE_INVESTMENTS = "CollectiveInvestments"
    CONSUMER_GOODS_CONGLOMERATES = "ConsumerGoodsConglomerates"
    CYCLICAL_CONSUMER_PRODUCTS = "CyclicalConsumerProducts"
    CYCLICAL_CONSUMER_SERVICES = "CyclicalConsumerServices"
    ENERGY_FOSSIL_FUELS = "EnergyFossilFuels"
    FINANCIAL_TECHNOLOGY_AND_INFRASTRUCTURE = "FinancialTechnologyAndInfrastructure"
    FOOD_AND_BEVERAGES = "FoodAndBeverages"
    FOOD_AND_DRUG_RETAILING = "FoodAndDrugRetailing"
    GOVERNMENT_ACTIVITY = "GovernmentActivity"
    HEALTHCARE_SERVICES_AND_EQUIPMENT = "HealthcareServicesAndEquipment"
    INDUSTRIAL_AND_COMMERCIAL_SERVICES = "IndustrialAndCommercialServices"
    INDUSTRIAL_GOODS = "IndustrialGoods"
    INSTITUTIONS_ASSOCIATIONS_AND_ORGANIZATIONS = (
        "InstitutionsAssociationsAndOrganizations"
    )
    INSURANCE = "Insurance"
    INVESTMENT_HOLDING_COMPANIES = "InvestmentHoldingCompanies"
    MINERAL_RESOURCES = "MineralResources"
    PERSONAL_AND_HOUSEHOLD_PRODUCTS_AND_SERVICES = (
        "PersonalAndHouseholdProductsAndServices"
    )
    PHARMACEUTICALS_AND_MEDICAL_RESEARCH = "PharmaceuticalsAndMedicalResearch"
    REAL_ESTATE = "RealEstate"
    RENEWABLE_ENERGY = "RenewableEnergy"
    RETAILERS = "Retailers"
    SOFTWARE_AND_IT_SERVICES = "SoftwareAndITServices"
    TECHNOLOGY_EQUIPMENT = "TechnologyEquipment"
    TELECOMMUNICATIONS_SERVICES = "TelecommunicationsServices"
    TRANSPORTATION = "Transportation"
    URANIUM = "Uranium"
    UTILITIES = "Utilities"



from dataclasses import dataclass
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..._types import OptStr, OptInt, OptBool, OptDict, OptStrings, OptDicts


@dataclass
class CFSBucket:
    attributes: "OptStrings" = None
    available_from: "OptStr" = None
    available_to: "OptStr" = None
    created: "OptStr" = None
    description: "OptStr" = None
    modified: "OptStr" = None
    name: "OptStr" = None
    private: "OptBool" = None
    publisher_name: "OptStr" = None
    restricted: "OptBool" = None

    def __iter__(self):
        return [CFSFileSet]


@dataclass
class CFSFileSet:
    attributes: "OptDicts" = None
    available_from: "OptStr" = None
    available_to: "OptStr" = None
    bucket_name: "OptStr" = None
    content_from: "OptStr" = None
    content_to: "OptStr" = None
    created: "OptStr" = None
    files: "OptStrings" = None
    id: "OptStr" = None
    modified: "OptStr" = None
    name: "OptStr" = None
    num_files: "OptInt" = None
    package_id: "OptStr" = None
    status: "OptStr" = None

    def __iter__(self):
        return [CFSFile]


@dataclass
class CFSFile:
    created: "OptStr" = None
    file_size_in_bytes: "OptInt" = None
    filename: "OptStr" = None
    fileset_id: "OptStr" = None
    href: "OptStr" = None
    id: "OptStr" = None
    modified: "OptStr" = None
    storage_location: "OptDict" = None


@dataclass
class CFSPackage:
    bucket_names: "OptStrings" = None
    claims: "OptDicts" = None
    contact_email: "OptStr" = None
    created: "OptStr" = None
    created_by: "OptStr" = None
    description: "OptStr" = None
    modified: "OptStr" = None
    modified_by: "OptStr" = None
    package_id: "OptStr" = None
    package_name: "OptStr" = None
    package_type: "OptStr" = None
    write_access_users: "OptStrings" = None
